<?php
/**
 * Class OrderController.
 *
 * @package ILabs\WoocommerceDHL24
 */

declare (strict_types=1);
namespace ILabs\WoocommerceDHL24\Controller;

use WP_Post;
use ILabs\WoocommerceDHL24\Helper\Param;
use ILabs\WoocommerceDHL24\ShippingMethod\WooCommerceShippingDHL24Parcel09;
use ILabs\WoocommerceDHL24\ShippingMethod\WooCommerceShippingDHL24Parcel12;
use ILabs\WoocommerceDHL24\ShippingMethod\WooCommerceShippingDHL24ParcelAH;
use ILabs\WoocommerceDHL24\ShippingMethod\WooCommerceShippingDHL24ParcelCM;
use ILabs\WoocommerceDHL24\ShippingMethod\WooCommerceShippingDHL24ParcelCP;
use ILabs\WoocommerceDHL24\ShippingMethod\WooCommerceShippingDHL24ParcelDW;
use ILabs\WoocommerceDHL24\ShippingMethod\WooCommerceShippingDHL24ParcelEK;
use ILabs\WoocommerceDHL24\ShippingMethod\WooCommerceShippingDHL24ParcelPI;
use ILabs\WoocommerceDHL24\ShippingMethod\WooCommerceShippingDHL24ParcelPR;
use ILabs\WoocommerceDHL24\ShippingMethod\WooCommerceShippingDHL24ServicePointLM;
use ILabs\WoocommerceDHL24\ShippingMethod\WooCommerceShippingDHL24ServicePointSP;

/**
 * OrderController
 */
class OrderController extends AbstractController implements ControllerInterface {

	/**
	 * Init class.
	 */
	public function init() {
		add_filter( 'woocommerce_shipping_methods', array( $this, 'add_shipping_method' ) );
		add_action( 'add_meta_boxes', array( $this, 'add_meta_boxes' ), 10, 2 );
	}

	/**
	 * Add shipping method
	 *
	 * @param  array $methods Methods.
	 * @return array
	 */
	public function add_shipping_method( $methods ) {
		$methods['dhl24_service_09'] = WooCommerceShippingDHL24Parcel09::class;
		$methods['dhl24_service_12'] = WooCommerceShippingDHL24Parcel12::class;
		$methods['dhl24_service_ah'] = WooCommerceShippingDHL24ParcelAH::class;
		$methods['dhl24_service_dw'] = WooCommerceShippingDHL24ParcelDW::class;
		$methods['dhl24_service_ek'] = WooCommerceShippingDHL24ParcelEK::class;
		$methods['dhl24_service_pi'] = WooCommerceShippingDHL24ParcelPI::class;
		$methods['dhl24_service_pr'] = WooCommerceShippingDHL24ParcelPR::class;
		$methods['dhl24_service_cp'] = WooCommerceShippingDHL24ParcelCP::class;
		$methods['dhl24_service_cm'] = WooCommerceShippingDHL24ParcelCM::class;
		$methods['dhl24_service_lm'] = WooCommerceShippingDHL24ServicePointLM::class;
		$methods['dhl24_service_sp'] = WooCommerceShippingDHL24ServicePointSP::class;
		return $methods;
	}

	/**
	 * Add meta boxes
	 *
	 * @param  string  $post_type Post type.
	 * @param  WP_Post $post Post object.
	 * @return void
	 */
	public function add_meta_boxes( string $post_type, WP_Post $post ) {
		if ( 'shop_order' === $post_type ) {
			$order                         = wc_get_order( $post->ID );
			$shipping_methods              = Param::get_instance()->get_value( 'shipping_methods' );
			$servicepoint_shipping_methods = Param::get_instance()->get_value( 'servicepoint_shipping_methods' );
			foreach ( $order->get_shipping_methods() as $shipping_method ) {
				if ( in_array( $shipping_method->get_method_id(), $shipping_methods, true ) ) {
					$is_servicepoint_label = in_array( $shipping_method->get_method_id(), $servicepoint_shipping_methods, true ) ? true : false;
					add_meta_box(
						'dhl24_meta-box-order',
						__( 'DHL24', 'woocommerce-dhl24' ) . $this->get_logo(),
						array( $this, 'meta_box_order' ),
						'shop_order',
						'side',
						'default',
						array(
							'is_servicepoint_label' => $is_servicepoint_label,
						)
					);
				}
			}
		}
	}

	/**
	 * Meta box display callback.
	 *
	 * @param  WP_Post $post Current post object.
	 * @param  array   $callback_args Callback data.
	 * @return void
	 */
	public function meta_box_order( WP_Post $post, $callback_args ) {
		$args                  = $callback_args['args'];
		$is_servicepoint_label = false;
		if ( isset( $args['is_servicepoint_label'] ) && true === $args['is_servicepoint_label'] ) {
			$is_servicepoint_label = true;
		}
		$return_shipments = (string) get_post_meta( $post->ID, 'dhl24_return_shipments', true );
		if ( empty( $return_shipments ) ) {
			$return_shipments = array();
		} else {
			$return_shipments = explode( ',', $return_shipments );
		}

		$this->render(
			'meta-box-order.php',
			array(
				'shipment_id'            => get_post_meta( $post->ID, 'dhl24_shipment_id', true ),
				'dispatch_number'        => get_post_meta( $post->ID, 'dhl24_dispatch_number', true ),
				'return_shipment_id'     => '',
				'return_shipments'       => $return_shipments,
				'return_dispatch_number' => get_post_meta( $post->ID, 'dhl24_return_dispatch_number', true ),
				'order_id'               => $post->ID,
				'is_servicepoint_label'  => $is_servicepoint_label,
			)
		);
	}

	/**
	 * Render HTML
	 *
	 * @param string $template_name Template name.
	 * @param array  $args Additional arguments passed to the template.
	 * @return void
	 */
	protected function render( string $template_name, array $args = array() ) {
		$template_name = 'admin/' . $template_name;

		parent::render( $template_name, $args );
	}

	/**
	 * Get logo
	 *
	 * @return string
	 */
	private function get_logo() {
		return '<img style="height:22px; float:right;" src="' . $this->plugin_options->get_plugin_url() . '/assets/images/dhl_logo.svg"/>';

	}
}
