<?php
/**
 * Class ShipmentInfo
 *
 * @package ILabs\WoocommerceDHL24
 */

namespace ILabs\WoocommerceDHL24\DHL24Client\ServicePoint\Type;

/**
 * ShipmentInfo
 */
class ShipmentInfo {
	/**
	 * Service type.
	 *
	 * @var string|null
	 */
	private $dropOffType;

	/**
	 * Type of shipping service.
	 * Dictionary value; possible values:
	 * LM - LM
	 * SP - SP
	 *
	 * @var string|null
	 */
	private $serviceType;

	/**
	 * Billing information.
	 *
	 * @var Billing|null
	 */
	private $billing;

	/**
	 * Ordering special services.
	 *
	 * @var \ArrayObject|Service[]
	 */
	private $specialServices;

	/**
	 * Shipping date.
	 *
	 * @var \DateTime
	 */
	private $shipmentDate;

	/**
	 * Beginning of the time range when a courier can collect the shipment; in HH:MM format.
	 *
	 * @var string|null
	 */
	private $shipmentStartHour;

	/**
	 * End of the time range when a courier can collect the shipment; in HH:MM format.
	 *
	 * @var string|null
	 */
	private $shipmentEndHour;

	/**
	 * Label type.
	 * Dictionary value; possible values:
	 * BLP - BLP
	 * ZBLP - ZBLP
	 *
	 * @var string|null
	 */
	private $labelType;

	/**
	 * Shipment number.
	 *
	 * @var string|null
	 */
	private $wayBill;

	/**
	 * Construct
	 *
	 * @return void
	 */
	public function __construct() {
		$this->specialServices = new \ArrayObject();
		$this->shipmentDate    = new \DateTime( 'tomorrow' );
	}

	/**
	 * Return associative array.
	 *
	 * @return array
	 */
	public function toArray(): array {
		$specialServices = array();
		foreach ( $this->getSpecialServices() as $item ) {
			$specialServices[] = $item->toArray();
		}
		return array(
			'dropOffType'       => $this->getDropOffType(),
			'serviceType'       => $this->getServiceType(),
			'billing'           => $this->getBilling() ? $this->getBilling()->toArray() : null,
			'specialServices'   => $specialServices,
			'shipmentDate'      => $this->getShipmentDate()->format( 'Y-m-d' ), // Date in YYYY-MM-DD format.
			'shipmentStartHour' => $this->getShipmentStartHour(),
			'shipmentEndHour'   => $this->getShipmentEndHour(),
			'labelType'         => $this->getLabelType(),
			'wayBill'           => $this->getWayBill(),
		);
	}

	/**
	 * Get Service type.
	 *
	 * @return string|null
	 */
	public function getDropOffType() {
		return $this->dropOffType;
	}

	/**
	 * Set Service type.
	 * Dictionary value; possible values:
	 * REGULAR_PICKUP - creating a shipment without limitations
	 * REQUEST_COURIER - creating a shipment and booking a courier
	 *
	 * @param string|null $dropOffType Service type.
	 *
	 * @return self
	 */
	public function setDropOffType( $dropOffType ) {
		$this->dropOffType = $dropOffType;

		return $this;
	}

	/**
	 * Get type of shipping service.
	 *
	 * @return string|null
	 */
	public function getServiceType() {
		return $this->serviceType;
	}

	/**
	 * Set type of shipping service.
	 *
	 * @param string|null $serviceType Type of shipping service.
	 *
	 * @return self
	 */
	public function setServiceType( $serviceType ) {
		$this->serviceType = $serviceType;

		return $this;
	}

	/**
	 * Get billing information.
	 *
	 * @return Billing|null
	 */
	public function getBilling() {
		return $this->billing;
	}

	/**
	 * Set billing information.
	 *
	 * @param Billing|null $billing Billing information.
	 *
	 * @return self
	 */
	public function setBilling( $billing ) {
		$this->billing = $billing;

		return $this;
	}

	/**
	 * Get ordering special services.
	 *
	 * @return \ArrayObject|Service[]
	 */
	public function getSpecialServices() {
		return $this->specialServices;
	}

	/**
	 * Add ordering special service.
	 *
	 * @param Service $specialService Ordering special service.
	 *
	 * @return self
	 */
	public function addSpecialServices( Service $specialService ) {
		$this->specialServices->append( $specialService );

		return $this;
	}

	/**
	 * Remove ordering special service.
	 *
	 * @param Service $specialService Ordering special service.
	 *
	 * @return self
	 */
	public function removeSpecialServices( Service $specialService ) {
		// TODO.

		return $this;
	}

	/**
	 * Get shipping date.
	 *
	 * @return  \DateTime
	 */
	public function getShipmentDate(): \DateTime {
		return $this->shipmentDate;
	}

	/**
	 * Set shipping date.
	 *
	 * @param  \DateTime $shipmentDate  Shipping date.
	 * @return  self
	 */
	public function setShipmentDate( \DateTime $shipmentDate ) {
		$this->shipmentDate = $shipmentDate;

		return $this;
	}

	/**
	 * Get beginning of the time range when a courier can collect the shipment; in HH:MM format.
	 *
	 * @return  string|null
	 */
	public function getShipmentStartHour() {
		return $this->shipmentStartHour;
	}

	/**
	 * Set beginning of the time range when a courier can collect the shipment; in HH:MM format.
	 *
	 * @param  string|null $shipmentStartHour  Beginning of the time range when a courier can collect the shipment; in HH:MM format.
	 *
	 * @return  self
	 */
	public function setShipmentStartHour( $shipmentStartHour ) {
		$this->shipmentStartHour = $shipmentStartHour;

		return $this;
	}

	/**
	 * Get end of the time range when a courier can collect the shipment; in HH:MM format.
	 *
	 * @return  string|null
	 */
	public function getShipmentEndHour() {
		return $this->shipmentEndHour;
	}

	/**
	 * Set end of the time range when a courier can collect the shipment; in HH:MM format.
	 *
	 * @param  string|null $shipmentEndHour  End of the time range when a courier can collect the shipment; in HH:MM format.
	 *
	 * @return  self
	 */
	public function setShipmentEndHour( $shipmentEndHour ) {
		$this->shipmentEndHour = $shipmentEndHour;

		return $this;
	}

	/**
	 * Get return label selection.
	 *
	 * @return string|null
	 */
	public function getLabelType() {
		return $this->labelType;
	}

	/**
	 * Set return label selection.
	 *
	 * @param string|null $labelType Return label selection.
	 *
	 * @return self
	 */
	public function setLabelType( $labelType ) {
		$this->labelType = $labelType;

		return $this;
	}

	/**
	 * Get shipment waybill number.
	 *
	 * @return string|null
	 */
	public function getWayBill() {
		return $this->wayBill;
	}

	/**
	 * Set shipment waybill number.
	 *
	 * @param string|null $wayBill Shipment waybill number.
	 *
	 * @return self
	 */
	public function setWayBill( $wayBill ) {
		$this->wayBill = $wayBill;

		return $this;
	}
}
