<?php
/**
 * Class CreateShipment
 *
 * @package ILabs\WoocommerceDHL24
 */

namespace ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Method;

use ILabs\WoocommerceDHL24\DHL24Client\RequestInterface;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\Ship;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\AuthData;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\CustomsData;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ShipmentInfo;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\PieceDefinition;

/**
 * CreateShipment
 */
class CreateShipment extends AbstractMethod implements RequestInterface {
	/**
	 * ShipmentInfo.
	 *
	 * @var ShipmentInfo|null
	 */
	private $shipmentInfo;

	/**
	 * Content of the parcel.
	 *
	 * @var string|null
	 */
	private $content;

	/**
	 * Additional comments to be printed on the consignment note.
	 *
	 * @var string|null
	 */
	private $comment;

	/**
	 * Shipment reference number.
	 *
	 * @var string|null
	 */
	private $reference;

	/**
	 * Ship.
	 *
	 * @var Ship|null
	 */
	private $ship;

	/**
	 * Parcel data, list of PieceDefinition elements.
	 *
	 * @var \ArrayObject|PieceDefinition[]
	 */
	private $pieceList;

	/**
	 * Data on customs clearance.
	 *
	 * @var CustomsData|null
	 */
	private $customs;

	/**
	 * Constructor
	 *
	 * @param AuthData $authData Authorization data.
	 */
	public function __construct( AuthData $authData ) {
		parent::__construct( $authData );

		$this->pieceList = new \ArrayObject();
	}

	/**
	 * Return associative array.
	 *
	 * @return array
	 */
	public function toArray(): array {
		$pieceList = array();
		foreach ( $this->getPieceList() as $key => $piece ) {
			$pieceList[] = $piece->toArray();
		}
		return array_merge(
			parent::toArray(),
			array(
				'shipment' => array(
					'shipmentInfo' => $this->getShipmentInfo() ? $this->getShipmentInfo()->toArray() : null,
					'content'      => $this->getContent(),
					'comment'      => $this->getComment(),
					'reference'    => $this->getReference(),
					'ship'         => $this->getShip() ? $this->getShip()->toArray() : null,
					'pieceList'    => $pieceList,
					'customs'      => $this->getCustoms() ? $this->getCustoms()->toArray() : null,
				),
			)
		);
	}

	/**
	 * Get shipmentInfo.
	 *
	 * @return ShipmentInfo|null
	 */
	public function getShipmentInfo() {
		return $this->shipmentInfo;
	}

	/**
	 * Set shipmentInfo.
	 *
	 * @param ShipmentInfo|null $shipmentInfo ShipmentInfo.
	 *
	 * @return self
	 */
	public function setShipmentInfo( $shipmentInfo ) {
		$this->shipmentInfo = $shipmentInfo;

		return $this;
	}

	/**
	 * Get content of the parcel.
	 *
	 * @return string|null
	 */
	public function getContent() {
		return $this->content;
	}

	/**
	 * Set content of the parcel.
	 *
	 * @param string|null $content Content of the parcel.
	 *
	 * @return self
	 */
	public function setContent( $content ) {
		$this->content = $content;

		return $this;
	}

	/**
	 * Get additional comments to be printed on the consignment note.
	 *
	 * @return string|null
	 */
	public function getComment() {
		return $this->comment;
	}

	/**
	 * Set additional comments to be printed on the consignment note.
	 *
	 * @param string|null $comment Additional comments to be printed on the consignment note.
	 *
	 * @return self
	 */
	public function setComment( $comment ) {
		$this->comment = $comment;

		return $this;
	}

	/**
	 * Get shipment reference number.
	 *
	 * @return string|null
	 */
	public function getReference() {
		return $this->reference;
	}

	/**
	 * Set shipment reference number.
	 *
	 * @param string|null $reference Shipment reference number.
	 *
	 * @return self
	 */
	public function setReference( $reference ) {
		$this->reference = $reference;

		return $this;
	}

	/**
	 * Get ship.
	 *
	 * @return Ship|null
	 */
	public function getShip() {
		return $this->ship;
	}

	/**
	 * Set ship.
	 *
	 * @param Ship|null $ship Ship.
	 *
	 * @return self
	 */
	public function setShip( $ship ) {
		$this->ship = $ship;

		return $this;
	}

	/**
	 * Get parcel data, list of PieceDefinition elements.
	 *
	 * @return \ArrayObject|PieceDefinition[]
	 */
	public function getPieceList() {
		return $this->pieceList;
	}

	/**
	 * Add piece.
	 *
	 * @param PieceDefinition $pieceDefinition PieceDefinition.
	 *
	 * @return self
	 */
	public function addPiece( PieceDefinition $pieceDefinition ) {
		$this->pieceList->append( $pieceDefinition );

		return $this;
	}

	/**
	 * Remove piece.
	 *
	 * @param PieceDefinition $pieceDefinition PieceDefinition.
	 *
	 * @return self
	 */
	public function removePiece( PieceDefinition $pieceDefinition ) {
		// TODO.

		return $this;
	}

	/**
	 * Get data on customs clearance.
	 *
	 * @return CustomsData|null
	 */
	public function getCustoms() {
		return $this->customs;
	}

	/**
	 * Set data on customs clearance.
	 *
	 * @param CustomsData|null $customs Data on customs clearance.
	 *
	 * @return self
	 */
	public function setCustoms( $customs ) {
		$this->customs = $customs;

		return $this;
	}
}
