<?php
/**
 * Class CustomsData
 *
 * @package ILabs\WoocommerceDHL24
 */

namespace ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type;

/**
 * CustomsData
 */
class CustomsData {
	/**
	 * Type of customs clearance (S - simplified, I - individual).
	 *
	 * @var string|null
	 */
	private $customsType;

	/**
	 * Name of the contact person.
	 *
	 * @var string|null
	 */
	private $firstName;

	/**
	 * Surname of the contact person.
	 *
	 * @var string|null
	 */
	private $secondaryName;

	/**
	 * Gross value of transport paid by the recipient to the buyer.
	 *
	 * @var string|null
	 */
	private $costsOfShipment;

	/**
	 * Currency (GBP, PLN, EUR, CHF, USD).
	 *
	 * @var string|null
	 */
	private $currency;

	/**
	 * NIP number.
	 *
	 * @var string|null
	 */
	private $nipNr;

	/**
	 * EORI number.
	 *
	 * @var string|null
	 */
	private $eoriNr;

	/**
	 * VAT registration number.
	 *
	 * @var string|null
	 */
	private $vatRegistrationNumber;

	/**
	 * Category of items (according to dictionary: 9 - “Other", 11 - “Sale of goods", 21 - “Return of goods", 31 - “Gifts", 32 - “Samples of goods", 91 - “Documents").
	 *
	 * @var string|null
	 */
	private $categoryOfItem;

	/**
	 * Invoice number.
	 *
	 * @var string|null
	 */
	private $invoiceNr;

	/**
	 * VAT invoice.
	 *
	 * @var string|null
	 */
	private $invoice;

	/**
	 * Invoice data (format YYYY-MM-DD).
	 *
	 * @var string|null
	 */
	private $invoiceDate;

	/**
	 * Country of origin of the goods. Permitted values according to ISO 3366-1 alfa-2.
	 *
	 * @var string|null
	 */
	private $countryOfOrigin;

	/**
	 * Additional information.
	 *
	 * @var string|null
	 */
	private $additionalInfo;

	/**
	 * Weight of the goods.
	 *
	 * @var string|null
	 */
	private $grossWeight;

	/**
	 * Array CustomsItemData structures.
	 *
	 * @var \ArrayObject|CustomsItem[]
	 */
	private $customsItems;

	/**
	 * Structure for consents and declarations.
	 *
	 * @var CustomsAgreementData|null
	 */
	private $customAgreements;

	/**
	 * Construct
	 *
	 * @return void
	 */
	public function __construct() {
		$this->customsItems = new \ArrayObject();
	}

	/**
	 * Return associative array.
	 *
	 * @return array
	 */
	public function toArray(): array {
		$customsItems = array();
		foreach ( $this->getCustomsItems() as $key => $customsItem ) {
			$customsItems[] = $customsItem->toArray();
		}

		return array(
			'customsType'           => $this->getCustomsType(),
			'firstName'             => $this->getFirstName(),
			'secondaryName'         => $this->getSecondaryName(),
			'costsOfShipment'       => $this->getCostsOfShipment(),
			'currency'              => $this->getCurrency(),
			'nipNr'                 => $this->getNipNr(),
			'eoriNr'                => $this->getEoriNr(),
			'vatRegistrationNumber' => $this->getVatRegistrationNumber(),
			'categoryOfItem'        => $this->getCategoryOfItem(),
			'invoiceNr'             => $this->getInvoiceNr(),
			'invoice'               => $this->getInvoice(),
			'invoiceDate'           => $this->getInvoiceDate(),
			'countryOfOrigin'       => $this->getCountryOfOrigin(),
			'additionalInfo'        => $this->getAdditionalInfo(),
			'grossWeight'           => $this->getGrossWeight(),
			'customsItem'           => $customsItems,
			'customAgreements'      => $this->getCustomAgreements()->toArray(),
		);
	}

	/**
	 * Get type of customs clearance (S - simplified, I - individual).
	 *
	 * @return string|null
	 */
	public function getCustomsType() {
		return $this->customsType;
	}

	/**
	 * Set type of customs clearance (S - simplified, I - individual).
	 *
	 * @param string|null $customsType Type of customs clearance (S - simplified, I - individual).
	 *
	 * @return self
	 */
	public function setCustomsType( $customsType ) {
		$this->customsType = $customsType;

		return $this;
	}

	/**
	 * Get name of the contact person.
	 *
	 * @return string|null
	 */
	public function getFirstName() {
		return $this->firstName;
	}

	/**
	 * Set name of the contact person.
	 *
	 * @param string|null $firstName Name of the contact person.
	 *
	 * @return self
	 */
	public function setFirstName( $firstName ) {
		$this->firstName = $firstName;

		return $this;
	}

	/**
	 * Get surname of the contact person.
	 *
	 * @return string|null
	 */
	public function getSecondaryName() {
		return $this->secondaryName;
	}

	/**
	 * Set surname of the contact person.
	 *
	 * @param string|null $secondaryName Surname of the contact person.
	 *
	 * @return self
	 */
	public function setSecondaryName( $secondaryName ) {
		$this->secondaryName = $secondaryName;

		return $this;
	}

	/**
	 * Get gross value of transport paid by the recipient to the buyer.
	 *
	 * @return string|null
	 */
	public function getCostsOfShipment() {
		return $this->costsOfShipment;
	}

	/**
	 * Set gross value of transport paid by the recipient to the buyer.
	 *
	 * @param string|null $costsOfShipment Gross value of transport paid by the recipient to the buyer.
	 *
	 * @return self
	 */
	public function setCostsOfShipment( $costsOfShipment ) {
		$this->costsOfShipment = $costsOfShipment;

		return $this;
	}

	/**
	 * Get currency (GBP, PLN, EUR, CHF, USD).
	 *
	 * @return string|null
	 */
	public function getCurrency() {
		return $this->currency;
	}

	/**
	 * Set currency (GBP, PLN, EUR, CHF, USD).
	 *
	 * @param string|null $currency Currency (GBP, PLN, EUR, CHF, USD).
	 *
	 * @return self
	 */
	public function setCurrency( $currency ) {
		$this->currency = $currency;

		return $this;
	}

	/**
	 * Get nIP number.
	 *
	 * @return string|null
	 */
	public function getNipNr() {
		return $this->nipNr;
	}

	/**
	 * Set nIP number.
	 *
	 * @param string|null $nipNr NIP number.
	 *
	 * @return self
	 */
	public function setNipNr( $nipNr ) {
		$this->nipNr = $nipNr;

		return $this;
	}

	/**
	 * Get eORI number.
	 *
	 * @return string|null
	 */
	public function getEoriNr() {
		return $this->eoriNr;
	}

	/**
	 * Set eORI number.
	 *
	 * @param string|null $eoriNr EORI number.
	 *
	 * @return self
	 */
	public function setEoriNr( $eoriNr ) {
		$this->eoriNr = $eoriNr;

		return $this;
	}

	/**
	 * Get vAT registration number.
	 *
	 * @return string|null
	 */
	public function getVatRegistrationNumber() {
		return $this->vatRegistrationNumber;
	}

	/**
	 * Set vAT registration number.
	 *
	 * @param string|null $vatRegistrationNumber VAT registration number.
	 *
	 * @return self
	 */
	public function setVatRegistrationNumber( $vatRegistrationNumber ) {
		$this->vatRegistrationNumber = $vatRegistrationNumber;

		return $this;
	}

	/**
	 * Get category of items (according to dictionary: 9 - “Other", 11 - “Sale of goods", 21 - “Return of goods", 31 - “Gifts", 32 - “Samples of goods", 91 - “Documents").
	 *
	 * @return string|null
	 */
	public function getCategoryOfItem() {
		return $this->categoryOfItem;
	}

	/**
	 * Set category of items (according to dictionary: 9 - “Other", 11 - “Sale of goods", 21 - “Return of goods", 31 - “Gifts", 32 - “Samples of goods", 91 - “Documents").
	 *
	 * @param string|null $categoryOfItem Category of items (according to dictionary: 9 - “Other", 11 - “Sale of goods", 21 - “Return of goods", 31 - “Gifts", 32 - “Samples of goods", 91 - “Documents").
	 *
	 * @return self
	 */
	public function setCategoryOfItem( $categoryOfItem ) {
		$this->categoryOfItem = $categoryOfItem;

		return $this;
	}

	/**
	 * Get invoice number.
	 *
	 * @return string|null
	 */
	public function getInvoiceNr() {
		return $this->invoiceNr;
	}

	/**
	 * Set invoice number.
	 *
	 * @param string|null $invoiceNr Invoice number.
	 *
	 * @return self
	 */
	public function setInvoiceNr( $invoiceNr ) {
		$this->invoiceNr = $invoiceNr;

		return $this;
	}

	/**
	 * Get vAT invoice.
	 *
	 * @return string|null
	 */
	public function getInvoice() {
		return $this->invoice;
	}

	/**
	 * Set vAT invoice.
	 *
	 * @param string|null $invoice VAT invoice.
	 *
	 * @return self
	 */
	public function setInvoice( $invoice ) {
		$this->invoice = $invoice;

		return $this;
	}

	/**
	 * Get invoice data (format YYYY-MM-DD).
	 *
	 * @return string|null
	 */
	public function getInvoiceDate() {
		return $this->invoiceDate;
	}

	/**
	 * Set invoice data (format YYYY-MM-DD).
	 *
	 * @param string|null $invoiceDate Invoice data (format YYYY-MM-DD).
	 *
	 * @return self
	 */
	public function setInvoiceDate( $invoiceDate ) {
		$this->invoiceDate = $invoiceDate;

		return $this;
	}

	/**
	 * Get country of origin of the goods. Permitted values according to ISO 3366-1 alfa-2.
	 *
	 * @return string|null
	 */
	public function getCountryOfOrigin() {
		return $this->countryOfOrigin;
	}

	/**
	 * Set country of origin of the goods. Permitted values according to ISO 3366-1 alfa-2.
	 *
	 * @param string|null $countryOfOrigin Country of origin of the goods. Permitted values according to ISO 3366-1 alfa-2.
	 *
	 * @return self
	 */
	public function setCountryOfOrigin( $countryOfOrigin ) {
		$this->countryOfOrigin = $countryOfOrigin;

		return $this;
	}

	/**
	 * Get additional information.
	 *
	 * @return string|null
	 */
	public function getAdditionalInfo() {
		return $this->additionalInfo;
	}

	/**
	 * Set additional information.
	 *
	 * @param string|null $additionalInfo Additional information.
	 *
	 * @return self
	 */
	public function setAdditionalInfo( $additionalInfo ) {
		$this->additionalInfo = $additionalInfo;

		return $this;
	}

	/**
	 * Get weight of the goods.
	 *
	 * @return string|null
	 */
	public function getGrossWeight() {
		return $this->grossWeight;
	}

	/**
	 * Set weight of the goods.
	 *
	 * @param string|null $grossWeight Weight of the goods.
	 *
	 * @return self
	 */
	public function setGrossWeight( $grossWeight ) {
		$this->grossWeight = $grossWeight;

		return $this;
	}

	/**
	 * Get array CustomsItemData structures.
	 *
	 * @return \ArrayObject|CustomsItem[]
	 */
	public function getCustomsItems(): \ArrayObject {
		return $this->customsItems;
	}

	/**
	 * Add CustomsItem structures.
	 *
	 * @param CustomsItem $customsItem CustomsItem.
	 *
	 * @return self
	 */
	public function addCustomsItem( CustomsItem $customsItem ) {
		$this->customsItems->append( $customsItem );

		return $this;
	}

	/**
	 * Remove CustomsItem structures.
	 *
	 * @param CustomsItem $customsItem CustomsItem.
	 *
	 * @return self
	 */
	public function removeCustomsItem( CustomsItem $customsItem ) {
		// TODO.

		return $this;
	}

	/**
	 * Get structure for consents and declarations.
	 *
	 * @return CustomsAgreementData|null
	 */
	public function getCustomAgreements() {
		return $this->customAgreements;
	}

	/**
	 * Set structure for consents and declarations.
	 *
	 * @param CustomsAgreementData|null $customAgreements Structure for consents and declarations.
	 *
	 * @return self
	 */
	public function setCustomAgreements( $customAgreements ) {
		$this->customAgreements = $customAgreements;

		return $this;
	}
}
