<?php
/**
 * Class Payment
 *
 * @package ILabs\WoocommerceDHL24
 */

namespace ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type;

/**
 * Payment
 */
class Payment {
	/**
	 * Payment method.
	 * Dictionary value:
	 * BANK_TRANSFER - bank transfer.
	 *
	 * @var string
	 */
	private $paymentMethod;

	/**
	 * Payer type.
	 * Dictionary value:
	 * SHIPPER - shipper is the payer
	 * USER - third party payer
	 *
	 * @var string
	 */
	private $payerType;

	/**
	 * Payer’s customer number (SAP), required for payerType = SHIPPER or USER.
	 *
	 * @var string
	 */
	private $accountNumber;

	/**
	 * CC field on the consignment note.
	 *
	 * @var string|null
	 */
	private $costsCenter;

	/**
	 * Construct
	 *
	 * @return void
	 */
	public function __construct() {
		$this->paymentMethod = 'BANK_TRANSFER';
		$this->payerType     = 'SHIPPER';
	}

	/**
	 * Return associative array.
	 *
	 * @return array
	 */
	public function toArray(): array {
		return array(
			'paymentMethod' => $this->getPaymentMethod(),
			'payerType'     => $this->getPayerType(),
			'accountNumber' => $this->getAccountNumber(),
			'costsCenter'   => $this->getCostsCenter(),
		);
	}

	/**
	 * Get payment method.
	 *
	 * @return string
	 */
	public function getPaymentMethod(): string {
		return $this->paymentMethod;
	}

	/**
	 * Set payment method.
	 *
	 * @param string $paymentMethod Payment method.
	 *
	 * @return self
	 */
	public function setPaymentMethod( string $paymentMethod ) {
		if ( in_array( $paymentMethod, array( 'BANK_TRANSFER' ), true ) ) {
			$this->paymentMethod = $paymentMethod;
		}

		return $this;
	}

	/**
	 * Get payer type.
	 *
	 * @return string
	 */
	public function getPayerType(): string {
		return $this->payerType;
	}

	/**
	 * Set payer type.
	 *
	 * @param string $payerType Payer type.
	 *
	 * @return self
	 */
	public function setPayerType( string $payerType ) {
		if ( in_array( $payerType, array( 'SHIPPER', 'USER' ), true ) ) {
			$this->payerType = $payerType;
		}

		return $this;
	}

	/**
	 * Get payer’s customer number (SAP), required for payerType = SHIPPER or USER.
	 *
	 * @return string
	 */
	public function getAccountNumber(): string {
		return $this->accountNumber;
	}

	/**
	 * Set payer’s customer number (SAP), required for payerType = SHIPPER or USER.
	 *
	 * @param string $accountNumber Payer’s customer number (SAP), required for payerType = SHIPPER or USER.
	 *
	 * @return self
	 */
	public function setAccountNumber( string $accountNumber ) {
		$this->accountNumber = $accountNumber;

		return $this;
	}

	/**
	 * Get CC field on the consignment note.
	 *
	 * @return string|null
	 */
	public function getCostsCenter() {
		return $this->costsCenter;
	}

	/**
	 * Set CC field on the consignment note.
	 *
	 * @param string|null $costsCenter CC field on the consignment note.
	 *
	 * @return self
	 */
	public function setCostsCenter( $costsCenter ) {
		$this->costsCenter = $costsCenter;

		return $this;
	}
}
