<?php
/**
 * Class ServiceDefinition
 *
 * @package ILabs\WoocommerceDHL24
 */

namespace ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type;

/**
 * ServiceDefinition
 */
class ServiceDefinition {

	/**
	 * Product.
	 * Dictionary value:
	 * AH - domestic shipment
	 * PR - Premium shipment
	 * 09 - Domestic 09
	 * 12 - Domestic 12
	 * DW - przesyłka z doręczeniem wieczornym
	 * EK - Connect shipment
	 * PI - International shipment
	 *
	 * @var string
	 */
	private $product;

	/**
	 * Selection of the Evening Delivery service.
	 *
	 * @var bool|null
	 */
	private $deliveryEvening;

	/**
	 * Selection of the Saturday Delivery service.
	 *
	 * @var bool|null
	 */
	private $deliveryOnSaturday;

	/**
	 * Selection of the Saturday Shipment service.
	 *
	 * @var bool|null
	 */
	private $pickupOnSaturday;

	/**
	 * Selection of the cash on delivery service.
	 *
	 * @var bool|null
	 */
	private $collectOnDelivery;

	/**
	 * Collection amount in PLN, accurate to within PLN 0.01, maximum amount PLN 11000.
	 *
	 * @var float|null
	 */
	private $collectOnDeliveryValue;

	/**
	 * Form of the collection amount refund.
	 * Permitted values: BANK_TRANSFER - bank transfer.
	 *
	 * @var string|null
	 */
	private $collectOnDeliveryForm;

	/**
	 * “COD reference” field on the consignment note.
	 *
	 * @var string|null
	 */
	private $collectOnDeliveryReference;

	/**
	 * Selection of the insurance service.
	 *
	 * @var bool|null
	 */
	private $insurance;

	/**
	 * Value of the insured shipment.
	 *
	 * @var float|null
	 */
	private $insuranceValue;

	/**
	 * Return of documents service.
	 *
	 * @var bool|null
	 */
	private $returnOnDelivery;

	/**
	 * Name of thereturn document.
	 *
	 * @var string|null
	 */
	private $returnOnDeliveryReference;

	/**
	 * Selection of the proof of delivery service.
	 * For a PI product- Selection of the disposal service.
	 *
	 * @var bool|null
	 */
	private $proofOfDelivery;

	/**
	 * Selection of the self-pickup service.
	 *
	 * @var bool|null
	 */
	private $selfCollect;

	/**
	 * Selection of the pre-delivery information service.
	 *
	 * @var bool|null
	 */
	private $predeliveryInformation;

	/**
	 * Delivery to neighbour.
	 *
	 * @var bool|null
	 */
	private $deliveryToNeighbour;

	/**
	 * Preaviso.
	 *
	 * @var bool|null
	 */
	private $preaviso;

	/**
	 * Additional service.
	 *
	 * @var bool|null
	 */
	private $additionalService;

	/**
	 * Construct
	 *
	 * @return void
	 */
	public function __construct() {
		$this->product = 'AH';
	}

	/**
	 * Return associative array.
	 *
	 * @return array
	 */
	public function toArray(): array {
		return array(
			'product'                    => $this->getProduct(),
			'deliveryEvening'            => $this->getDeliveryEvening(),
			'deliveryOnSaturday'         => $this->getDeliveryOnSaturday(),
			'pickupOnSaturday'           => $this->getPickupOnSaturday(),
			'collectOnDelivery'          => $this->getCollectOnDelivery(),
			'collectOnDeliveryValue'     => $this->getCollectOnDeliveryValue(),
			'collectOnDeliveryForm'      => $this->getCollectOnDeliveryForm(),
			'collectOnDeliveryReference' => $this->getCollectOnDeliveryReference(),
			'insurance'                  => $this->getInsurance(),
			'insuranceValue'             => $this->getInsuranceValue(),
			'returnOnDelivery'           => $this->getReturnOnDelivery(),
			'returnOnDeliveryReference'  => $this->getReturnOnDeliveryReference(),
			'proofOfDelivery'            => $this->getProofOfDelivery(),
			'selfCollect'                => $this->getSelfCollect(),
			'predeliveryInformation'     => $this->getPredeliveryInformation(),
			'deliveryToNeighbour'        => $this->getDeliveryToNeighbour(),
			'preaviso'                   => $this->getPreaviso(),
			'additionalService'          => $this->getAdditionalService(),
		);
	}

	/**
	 * Get product.
	 *
	 * @return string
	 */
	public function getProduct(): string {
		return $this->product;
	}

	/**
	 * Set product.
	 *
	 * @param string $product Product.
	 *
	 * @return self
	 */
	public function setProduct( string $product ) {
		if ( in_array( $product, array( 'AH', 'PR', '09', '12', 'DW', 'EK', 'PI' ), true ) ) {
			$this->product = $product;
		}

		return $this;
	}

	/**
	 * Get selection of the Evening Delivery service.
	 *
	 * @return bool|null
	 */
	public function getDeliveryEvening() {
		return $this->deliveryEvening;
	}

	/**
	 * Set selection of the Evening Delivery service.
	 *
	 * @param bool|null $deliveryEvening Selection of the Evening Delivery service.
	 *
	 * @return self
	 */
	public function setDeliveryEvening( $deliveryEvening ) {
		$this->deliveryEvening = $deliveryEvening;

		return $this;
	}

	/**
	 * Get selection of the Saturday Delivery service.
	 *
	 * @return bool|null
	 */
	public function getDeliveryOnSaturday() {
		return $this->deliveryOnSaturday;
	}

	/**
	 * Set selection of the Saturday Delivery service.
	 *
	 * @param bool|null $deliveryOnSaturday Selection of the Saturday Delivery service.
	 *
	 * @return self
	 */
	public function setDeliveryOnSaturday( $deliveryOnSaturday ) {
		$this->deliveryOnSaturday = $deliveryOnSaturday;

		return $this;
	}

	/**
	 * Get selection of the Saturday Shipment service.
	 *
	 * @return bool|null
	 */
	public function getPickupOnSaturday() {
		return $this->pickupOnSaturday;
	}

	/**
	 * Set selection of the Saturday Shipment service.
	 *
	 * @param bool|null $pickupOnSaturday Selection of the Saturday Shipment service.
	 *
	 * @return self
	 */
	public function setPickupOnSaturday( $pickupOnSaturday ) {
		$this->pickupOnSaturday = $pickupOnSaturday;

		return $this;
	}

	/**
	 * Get selection of the cash on delivery service.
	 *
	 * @return bool|null
	 */
	public function getCollectOnDelivery() {
		return $this->collectOnDelivery;
	}

	/**
	 * Set selection of the cash on delivery service.
	 *
	 * @param bool|null $collectOnDelivery Selection of the cash on delivery service.
	 *
	 * @return self
	 */
	public function setCollectOnDelivery( $collectOnDelivery ) {
		$this->collectOnDelivery = $collectOnDelivery;

		return $this;
	}

	/**
	 * Get collection amount in PLN, accurate to within PLN 0.01, maximum amount PLN 11000.
	 *
	 * @return float|null
	 */
	public function getCollectOnDeliveryValue() {
		if ( null !== $this->collectOnDeliveryValue ) {
			return round( $this->collectOnDeliveryValue, 2 );
		}

		return null;
	}

	/**
	 * Set collection amount in PLN, accurate to within PLN 0.01, maximum amount PLN 11000.
	 *
	 * @param float|null $collectOnDeliveryValue Collection amount in PLN, accurate to within PLN 0.01, maximum amount PLN 11000.
	 *
	 * @return self
	 */
	public function setCollectOnDeliveryValue( $collectOnDeliveryValue ) {
		if ( null === $collectOnDeliveryValue || ( 0.01 <= $collectOnDeliveryValue && 11000 >= $collectOnDeliveryValue ) ) {
			$this->collectOnDeliveryValue = $collectOnDeliveryValue;
		}

		return $this;
	}

	/**
	 * Get permitted values: BANK_TRANSFER - bank transfer.
	 *
	 * @return string|null
	 */
	public function getCollectOnDeliveryForm() {
		return $this->collectOnDeliveryForm;
	}

	/**
	 * Set permitted values: BANK_TRANSFER - bank transfer.
	 *
	 * @param string|null $collectOnDeliveryForm Permitted values: BANK_TRANSFER - bank transfer.
	 *
	 * @return self
	 */
	public function setCollectOnDeliveryForm( $collectOnDeliveryForm ) {
		if ( null === $collectOnDeliveryForm || in_array( $collectOnDeliveryForm, array( 'BANK_TRANSFER' ), true ) ) {
			$this->collectOnDeliveryForm = $collectOnDeliveryForm;
		}

		return $this;
	}

	/**
	 * Get “COD reference” field on the consignment note.
	 *
	 * @return string|null
	 */
	public function getCollectOnDeliveryReference() {
		return $this->collectOnDeliveryReference;
	}

	/**
	 * Set “COD reference” field on the consignment note.
	 *
	 * @param string|null $collectOnDeliveryReference “COD reference” field on the consignment note.
	 *
	 * @return self
	 */
	public function setCollectOnDeliveryReference( $collectOnDeliveryReference ) {
		$this->collectOnDeliveryReference = $collectOnDeliveryReference;

		return $this;
	}

	/**
	 * Get selection of the insurance service.
	 *
	 * @return bool|null
	 */
	public function getInsurance() {
		return $this->insurance;
	}

	/**
	 * Set selection of the insurance service.
	 *
	 * @param bool|null $insurance Selection of the insurance service.
	 *
	 * @return self
	 */
	public function setInsurance( $insurance ) {
		$this->insurance = $insurance;

		return $this;
	}

	/**
	 * Get value of the insured shipment.
	 *
	 * @return float|null
	 */
	public function getInsuranceValue() {
		if ( null !== $this->insuranceValue ) {
			return round( $this->insuranceValue, 2 );
		}
		return null;
	}

	/**
	 * Set value of the insured shipment.
	 *
	 * @param float|null $insuranceValue Value of the insured shipment.
	 *
	 * @return self
	 */
	public function setInsuranceValue( $insuranceValue ) {
		$this->insuranceValue = $insuranceValue;

		return $this;
	}

	/**
	 * Getreturn of documents service.
	 *
	 * @return bool|null
	 */
	public function getReturnOnDelivery() {
		return $this->returnOnDelivery;
	}

	/**
	 * Setreturn of documents service.
	 *
	 * @param bool|null $returnOnDelivery Return of documents service.
	 *
	 * @return self
	 */
	public function setReturnOnDelivery( $returnOnDelivery ) {
		$this->returnOnDelivery = $returnOnDelivery;

		return $this;
	}

	/**
	 * Get name of thereturn document.
	 *
	 * @return string|null
	 */
	public function getReturnOnDeliveryReference() {
		return $this->returnOnDeliveryReference;
	}

	/**
	 * Set name of thereturn document.
	 *
	 * @param string|null $returnOnDeliveryReference Name of thereturn document.
	 *
	 * @return self
	 */
	public function setReturnOnDeliveryReference( $returnOnDeliveryReference ) {
		$this->returnOnDeliveryReference = $returnOnDeliveryReference;

		return $this;
	}

	/**
	 * Get for a PI product- Selection of the disposal service.
	 *
	 * @return bool|null
	 */
	public function getProofOfDelivery() {
		return $this->proofOfDelivery;
	}

	/**
	 * Set for a PI product- Selection of the disposal service.
	 *
	 * @param bool|null $proofOfDelivery For a PI product- Selection of the disposal service.
	 *
	 * @return self
	 */
	public function setProofOfDelivery( $proofOfDelivery ) {
		$this->proofOfDelivery = $proofOfDelivery;

		return $this;
	}

	/**
	 * Get selection of the self-pickup service.
	 *
	 * @return bool|null
	 */
	public function getSelfCollect() {
		return $this->selfCollect;
	}

	/**
	 * Set selection of the self-pickup service.
	 *
	 * @param bool|null $selfCollect Selection of the self-pickup service.
	 *
	 * @return self
	 */
	public function setSelfCollect( $selfCollect ) {
		$this->selfCollect = $selfCollect;

		return $this;
	}

	/**
	 * Get selection of the pre-delivery information service.
	 *
	 * @return bool|null
	 */
	public function getPredeliveryInformation() {
		return $this->predeliveryInformation;
	}

	/**
	 * Set selection of the pre-delivery information service.
	 *
	 * @param bool|null $predeliveryInformation Selection of the pre-delivery information service.
	 *
	 * @return self
	 */
	public function setPredeliveryInformation( $predeliveryInformation ) {
		$this->predeliveryInformation = $predeliveryInformation;

		return $this;
	}

	/**
	 * Get delivery to neighbour.
	 *
	 * @return bool|null
	 */
	public function getDeliveryToNeighbour() {
		return $this->deliveryToNeighbour;
	}

	/**
	 * Set delivery to neighbour.
	 *
	 * @param bool|null $deliveryToNeighbour Delivery to neighbour.
	 *
	 * @return self
	 */
	public function setDeliveryToNeighbour( $deliveryToNeighbour ) {
		$this->deliveryToNeighbour = $deliveryToNeighbour;

		return $this;
	}

	/**
	 * Get preaviso.
	 *
	 * @return bool|null
	 */
	public function getPreaviso() {
		return $this->preaviso;
	}

	/**
	 * Set preaviso.
	 *
	 * @param bool|null $preaviso Preaviso.
	 *
	 * @return self
	 */
	public function setPreaviso( $preaviso ) {
		$this->preaviso = $preaviso;

		return $this;
	}

	/**
	 * Get additional service.
	 *
	 * @return bool|null
	 */
	public function getAdditionalService() {
		return $this->additionalService;
	}

	/**
	 * Set additional service.
	 *
	 * @param bool|null $additionalService Additional service.
	 *
	 * @return self
	 */
	public function setAdditionalService( $additionalService ) {
		$this->additionalService = $additionalService;

		return $this;
	}
}
