<?php
/**
 * Class ShipmentInfo
 *
 * @package ILabs\WoocommerceDHL24
 */

namespace ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type;

/**
 * ShipmentInfo
 */
class ShipmentInfo {
	/**
	 * Shipment waybill number.
	 *
	 * @var string|null
	 */
	private $wayBill;

	/**
	 * Request type.
	 * Dictionary value; possible values:
	 * REGULAR_PICKUP - creating a shipment without limitations
	 * REQUEST_COURIER - creating a shipment and booking a courier
	 * COURIER_ONLY - booking a courier
	 *
	 * @var string|null
	 */
	private $dropOffType;

	/**
	 * Type of shipping service.
	 * Dictionary value; possible values:
	 * AH - domestic shipment
	 * 09 - Domestic service 09
	 * 12 - Domestic service 12
	 * DW - evening delivery shipment
	 * EK - Connect shipment
	 * PI - International shipment
	 * PR - Premium product
	 *
	 * @var string|null
	 */
	private $serviceType;

	/**
	 * Billing information.
	 *
	 * @var Billing|null
	 */
	private $billing;

	/**
	 * Ordering special services.
	 *
	 * @var \ArrayObject|Service[]
	 */
	private $specialServices;

	/**
	 * Information on the parcel shipment time.
	 *
	 * @var ShipmentTime|null
	 */
	private $shipmentTime;

	/**
	 * Return label selection.
	 *
	 * @var string|null
	 */
	private $labelType;

	/**
	 * Return associative array.
	 *
	 * @return array
	 */
	public function toArray(): array {
		$specialServices = array();
		foreach ( $this->getSpecialServices() as $item ) {
			$specialServices[] = $item->toArray();
		}
		return array(
			'wayBill'         => $this->getWayBill(),
			'dropOffType'     => $this->getDropOffType(),
			'serviceType'     => $this->getServiceType(),
			'billing'         => $this->getBilling() ? $this->getBilling()->toArray() : null,
			'specialServices' => $specialServices,
			'shipmentTime'    => $this->getShipmentTime() ? $this->getShipmentTime()->toArray() : null,
			'labelType'       => $this->getLabelType(),
		);
	}

	/**
	 * Construct
	 *
	 * @return void
	 */
	public function __construct() {
		$this->specialServices = new \ArrayObject();
	}

	/**
	 * Get shipment waybill number.
	 *
	 * @return string|null
	 */
	public function getWayBill() {
		return $this->wayBill;
	}

	/**
	 * Set shipment waybill number.
	 *
	 * @param string|null $wayBill Shipment waybill number.
	 *
	 * @return self
	 */
	public function setWayBill( $wayBill ) {
		$this->wayBill = $wayBill;

		return $this;
	}

	/**
	 * Get request type.
	 *
	 * @return string|null
	 */
	public function getDropOffType() {
		return $this->dropOffType;
	}

	/**
	 * Set request type.
	 *
	 * @param string|null $dropOffType Request type.
	 *
	 * @return self
	 */
	public function setDropOffType( $dropOffType ) {
		$this->dropOffType = $dropOffType;

		return $this;
	}

	/**
	 * Get type of shipping service.
	 *
	 * @return string|null
	 */
	public function getServiceType() {
		return $this->serviceType;
	}

	/**
	 * Set type of shipping service.
	 *
	 * @param string|null $serviceType Type of shipping service.
	 *
	 * @return self
	 */
	public function setServiceType( $serviceType ) {
		$this->serviceType = $serviceType;

		return $this;
	}

	/**
	 * Get billing information.
	 *
	 * @return Billing|null
	 */
	public function getBilling() {
		return $this->billing;
	}

	/**
	 * Set billing information.
	 *
	 * @param Billing|null $billing Billing information.
	 *
	 * @return self
	 */
	public function setBilling( $billing ) {
		$this->billing = $billing;

		return $this;
	}

	/**
	 * Get ordering special services.
	 *
	 * @return \ArrayObject|Service[]
	 */
	public function getSpecialServices() {
		return $this->specialServices;
	}

	/**
	 * Add ordering special service.
	 *
	 * @param Service $specialService Ordering special service.
	 *
	 * @return self
	 */
	public function addSpecialServices( Service $specialService ) {
		$this->specialServices->append( $specialService );

		return $this;
	}

	/**
	 * Remove ordering special service.
	 *
	 * @param Service $specialService Ordering special service.
	 *
	 * @return self
	 */
	public function removeSpecialServices( Service $specialService ) {
		// TODO.

		return $this;
	}

	/**
	 * Get information on the parcel shipment time.
	 *
	 * @return ShipmentTime|null
	 */
	public function getShipmentTime() {
		return $this->shipmentTime;
	}

	/**
	 * Set information on the parcel shipment time.
	 *
	 * @param ShipmentTime|null $shipmentTime Information on the parcel shipment time.
	 *
	 * @return self
	 */
	public function setShipmentTime( $shipmentTime ) {
		$this->shipmentTime = $shipmentTime;

		return $this;
	}

	/**
	 * Get return label selection.
	 *
	 * @return string|null
	 */
	public function getLabelType() {
		return $this->labelType;
	}

	/**
	 * Set return label selection.
	 *
	 * @param string|null $labelType Return label selection.
	 *
	 * @return self
	 */
	public function setLabelType( $labelType ) {
		$this->labelType = $labelType;

		return $this;
	}
}
