<?php
/**
 * Class AdminAjax.
 *
 * @package ILabs\WoocommerceDHL24
 */

namespace ILabs\WoocommerceDHL24\Helper;

use ILabs\WoocommerceDHL24\Helper\Param;
use ILabs\WoocommerceDHL24\Exception\ExceptionInterface;
use ILabs\WoocommerceDHL24\Helper\CreateShipmentContext;
use ILabs\WoocommerceDHL24\Repository\ShipmentRepository;
use ILabs\WoocommerceDHL24\Repository\ShipmentsRepository;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\LabelsData;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\LabelsDatas;

/**
 * AdminAjax
 */
class AdminAjax {

	/**
	 * Normalizer
	 *
	 * @var Normalizer
	 */
	private $normalizer;

	/**
	 * Construct
	 *
	 * @return void
	 */
	public function __construct() {
		$this->normalizer = new Normalizer();
	}

	/**
	 * Order courier
	 *
	 * @param  string|string[] $data Unslashed $data.
	 * @return bool
	 */
	public function order_courier( $data ): bool {
		if ( ! empty( $data['_nonce'] ) && wp_verify_nonce( sanitize_text_field( $data['_nonce'] ), 'action_order_courier' ) ) {
			$ids         = ! empty( $data['ids'] ) ? sanitize_text_field( $data['ids'] ) : '';
			$pickup_date = ! empty( $data['pickup_date'] ) ? $this->normalizer->date( $data['pickup_date'] ) : null;
			$from        = ! empty( $data['from'] ) ? $this->normalizer->hour( $data['from'] ) : '';
			$to          = ! empty( $data['to'] ) ? $this->normalizer->hour( $data['to'] ) : '';
			$comment     = ! empty( $data['comment'] ) ? sanitize_text_field( $data['comment'] ) : '';
			$label       = ! empty( $data['label'] ) ? $this->normalizer->bool( $data['label'] ) : false;
			if ( '' === $ids || null === $pickup_date || '' === $from || '' === $to ) {
				wp_send_json( array( __( 'Invalid data.', 'woocommerce-dhl24' ) ), \WP_Http::BAD_REQUEST );
			}
			$ids = explode( ',', $ids );

			$shipments_repository = new ShipmentsRepository();

			$courier_ids = new \ArrayObject();
			try {
				$courier_ids = $shipments_repository->book_courier( $ids, $pickup_date, $from, $to, $comment, $label );
				$courier_ids = $courier_ids->getItems();
			} catch ( ExceptionInterface $e ) {
				wp_send_json( array( esc_attr( $e->getMessage() ) ), \WP_Http::UNPROCESSABLE_ENTITY );
			}

			$shipment = new Shipment();
			foreach ( $ids as $key => $id ) {
				$order_id = $shipment->find_order_id( 'dhl24_shipment_id', $id );
				if ( ! empty( $order_id ) ) {
					update_post_meta( $order_id, 'dhl24_dispatch_number', $courier_ids[ $key ] );
				}
				$order_id = $shipment->find_order_id( 'dhl24_return_shipments', $id );
				if ( ! empty( $order_id ) ) {
					$return_shipments        = (string) get_post_meta( $order_id, 'dhl24_return_shipments', true );
					$return_shipments        = explode( ',', $return_shipments );
					$return_dispatch_numbers = (string) get_post_meta( $order_id, 'dhl24_return_dispatch_numbers', true );
					$return_dispatch_numbers = explode( ',', $return_dispatch_numbers );
					$dispatch_numbers        = array();
					foreach ( $return_shipments as $k => $value ) {
						if ( $value == $id ) {
							$dispatch_numbers[ $k ] = $courier_ids[ $key ];
						} else {
							$dispatch_numbers[ $k ] = $return_dispatch_numbers[ $k ];
						}
					}
					update_post_meta( $order_id, 'dhl24_return_dispatch_numbers', implode( ',', $dispatch_numbers ) );
				}
			}

			wp_send_json( $courier_ids, \WP_Http::OK );
		}

		return false;
	}

	/**
	 * New shipment
	 *
	 * @param  string|string[] $data Unslashed $data.
	 * @return bool
	 */
	public function new_shipment( $data ): bool {
		if ( ! empty( $data['_nonce'] ) && wp_verify_nonce( sanitize_text_field( $data['_nonce'] ), 'ajax_new_shipment' ) ) {
			$shipping_method = ! empty( $data['shipping_method'] ) ? sanitize_text_field( $data['shipping_method'] ) : '';
			$order_id        = ! empty( $data['order_id'] ) ? sanitize_text_field( $data['order_id'] ) : '';
			$order           = wc_get_order( $order_id );

			if ( false === $order ) {
				wp_send_json( array( __( 'Invalid data.', 'woocommerce-dhl24' ) ), \WP_Http::BAD_REQUEST );
			}
			$data['shipping_company']    = $order->get_shipping_company();
			$data['shipping_first_name'] = $order->get_shipping_first_name();
			$data['shipping_last_name']  = $order->get_shipping_last_name();
			$data['shipping_address_1']  = $order->get_shipping_address_1();
			$data['shipping_address_2']  = $order->get_shipping_address_2();
			$data['shipping_country']    = $order->get_shipping_country();
			$data['shipping_postcode']   = $order->get_shipping_postcode();
			$data['shipping_city']       = $order->get_shipping_city();
			$data['billing_phone']       = $order->get_billing_phone();
			$data['billing_email']       = $order->get_billing_email();

			$data['shipping_method'] = strlen( $shipping_method ) > 2 ? strtoupper( substr( $shipping_method, -2 ) ) : '';
			$shipping_methods        = Param::get_instance()->get_value( 'servicepoint_shipping_methods' );
			if ( in_array( $shipping_method, $shipping_methods, true ) ) {
				$data['dhl24_sap'] = $order->get_meta( 'dhl24_sap' );
			};

			if ( ! empty( ( $data['service'] ) ) && is_array( $data['service'] ) ) {
				$data['service']['insurance'] = ! empty( $data['service']['insurance'] ) ? $this->normalizer->bool( $data['service']['insurance'] ) : false;
				if ( false === $data['service']['insurance'] ) {
					$data['service']['insuranceValue'] = '';
				} else {
					$data['service']['insuranceValue'] = ! empty( $data['service']['insuranceValue'] ) ? $this->normalizer->float( $data['service']['insuranceValue'] ) : '';
				}
				$data['service']['collectOnDelivery'] = ! empty( $data['service']['collectOnDelivery'] ) ? $this->normalizer->bool( $data['service']['collectOnDelivery'] ) : false;
				if ( false === $data['service']['collectOnDelivery'] ) {
					$data['service']['collectOnDeliveryValue'] = '';
				} else {
					$data['service']['collectOnDeliveryValue'] = ! empty( $data['service']['collectOnDeliveryValue'] ) ? $this->normalizer->float( $data['service']['collectOnDeliveryValue'] ) : '';
				}
				$data['service']['predeliveryInformation'] = ! empty( $data['service']['predeliveryInformation'] ) ? $this->normalizer->bool( $data['service']['predeliveryInformation'] ) : false;
				$data['service']['proofOfDelivery']        = ! empty( $data['service']['proofOfDelivery'] ) ? $this->normalizer->bool( $data['service']['proofOfDelivery'] ) : false;
				$data['service']['returnOnDelivery']       = ! empty( $data['service']['returnOnDelivery'] ) ? $this->normalizer->bool( $data['service']['returnOnDelivery'] ) : false;
				if ( false === $data['service']['returnOnDelivery'] ) {
					$data['service']['returnOnDeliveryReference'] = '';
				} else {
					$data['service']['returnOnDeliveryReference'] = ! empty( $data['service']['returnOnDeliveryReference'] ) ? sanitize_text_field( $data['service']['returnOnDeliveryReference'] ) : '';
				}
				$data['service']['selfCollect']        = ! empty( $data['service']['selfCollect'] ) ? $this->normalizer->bool( $data['service']['selfCollect'] ) : false;
				$data['service']['deliveryOnSaturday'] = ! empty( $data['service']['deliveryOnSaturday'] ) ? $this->normalizer->bool( $data['service']['deliveryOnSaturday'] ) : false;
			}

			if ( ! empty( ( $data['piece'] ) ) && is_array( $data['piece'] ) ) {
				$pieces = array();
				foreach ( $data['piece'] as $piece ) {
					if ( ! empty( ( $piece ) ) && is_array( $piece ) ) {
						$pieces[] = array(
							'quantity'    => ! empty( $piece['quantity'] ) ? $this->normalizer->integer( $piece['quantity'] ) : null,
							'type'        => ! empty( $piece['type'] ) ? $this->normalizer->piece_type( $piece['type'] ) : null,
							'weight'      => ! empty( $piece['weight'] ) ? $this->normalizer->float( $piece['weight'] ) : null,
							'length'      => ! empty( $piece['length'] ) ? $this->normalizer->integer( $piece['length'] ) : null,
							'width'       => ! empty( $piece['width'] ) ? $this->normalizer->integer( $piece['width'] ) : null,
							'height'      => ! empty( $piece['height'] ) ? $this->normalizer->integer( $piece['height'] ) : null,
							'nonStandard' => ! empty( $piece['nonStandard'] ) ? $this->normalizer->bool( $piece['nonStandard'] ) : null,
							'euroReturn'  => ! empty( $piece['euroReturn'] ) ? $this->normalizer->bool( $piece['euroReturn'] ) : null,
						);
					}
				}
				$data['piece'] = $pieces;
			}

			$data['content']       = ! empty( $data['content'] ) ? sanitize_text_field( $data['content'] ) : '';
			$data['comment']       = ! empty( $data['comment'] ) ? sanitize_text_field( $data['comment'] ) : '';
			$data['reference']     = ! empty( $data['reference'] ) ? sanitize_text_field( $data['reference'] ) : '';
			$data['costs_center']  = ! empty( $data['costs_center'] ) ? sanitize_text_field( $data['costs_center'] ) : '';
			$data['shipment_date'] = ! empty( $data['shipment_date'] ) ? $this->normalizer->date( $data['shipment_date'] ) : null;

			// Validate.
			$is_error = false;
			if ( empty( $data['content'] ) ) {
				Notice::get_instance()->add( 'error', __( 'Field "Content" is required.', 'woocommerce-dhl24' ) );
				$is_error = true;
			}
			if ( true === $is_error ) {
				wp_send_json( Notice::get_instance()->get_notices(), \WP_Http::UNPROCESSABLE_ENTITY );
			}

			$create_shipment = new CreateShipmentContext( $shipping_method, $order_id, $data );

			try {
				$shipment = $create_shipment->create();
			} catch ( ExceptionInterface $e ) {
				wp_send_json( array( esc_attr( $e->getMessage() ) ), \WP_Http::UNPROCESSABLE_ENTITY );
			}

			$this->send_label( $shipment['fileName'], $shipment['mimeType'], $shipment['fileData'], $order_id );
		}

		return false;
	}

	/**
	 * Return shipment
	 *
	 * @param  string|string[] $data Unslashed $data.
	 * @return bool
	 */
	public function return_shipment( $data ): bool {
		if ( ! empty( $data['_nonce'] ) && wp_verify_nonce( sanitize_text_field( $data['_nonce'] ), 'ajax_return_shipment' ) ) {
			$shipment_id     = ! empty( $data['shipment_id'] ) ? sanitize_text_field( $data['shipment_id'] ) : '';
			$shipping_method = ! empty( $data['shipping_method'] ) ? sanitize_text_field( $data['shipping_method'] ) : '';
			$order_id        = ! empty( $data['order_id'] ) ? $this->normalizer->integer( $data['order_id'] ) : 0;

			if ( empty( $shipment_id ) ) {
				wp_send_json( array( __( 'Invalid data.', 'woocommerce-dhl24' ) ), \WP_Http::BAD_REQUEST );
			}

			$shipment_repository = new ShipmentRepository();

			try {
				$labels = $shipment_repository->fetch_label( $shipment_id );
			} catch ( ExceptionInterface $e ) {
				wp_send_json( array( $e->getMessage() ), \WP_Http::BAD_REQUEST );
			}
			if ( 0 === $labels->getItems()->count() ) {
				wp_send_json( array( __( 'Invalid data.', 'woocommerce-dhl24' ) ), \WP_Http::BAD_REQUEST );
			}
			/**
			 * LabelsData
			 *
			 * @var LabelsData $labels_data
			 */
			$labels_data     = $labels->getItems()->offsetGet( 0 );
			$data['shipper'] = $labels_data->getReceiver();

			unset( $data['shipment_id'] );
			$data['primary_waybill_number'] = $shipment_id;
			$data['shipping_method']        = strlen( $shipping_method ) > 2 ? strtoupper( substr( $shipping_method, -2 ) ) : '';

			if ( ! empty( ( $data['service'] ) ) && is_array( $data['service'] ) ) {
				$data['service']['insurance'] = ! empty( $data['service']['insurance'] ) ? $this->normalizer->bool( $data['service']['insurance'] ) : false;
				if ( false === $data['service']['insurance'] ) {
					$data['service']['insuranceValue'] = '';
				} else {
					$data['service']['insuranceValue'] = ! empty( $data['service']['insuranceValue'] ) ? sanitize_text_field( $data['service']['insuranceValue'] ) : '';
				}
			}

			if ( ! empty( ( $data['piece'] ) ) && is_array( $data['piece'] ) ) {
				$pieces = array();
				foreach ( $data['piece'] as $piece ) {
					if ( ! empty( ( $piece ) ) && is_array( $piece ) ) {
						$pieces[] = array(
							'quantity'    => ! empty( $piece['quantity'] ) ? $this->normalizer->integer( $piece['quantity'] ) : null,
							'type'        => ! empty( $piece['type'] ) ? $this->normalizer->piece_type( $piece['type'] ) : null,
							'weight'      => ! empty( $piece['weight'] ) ? $this->normalizer->float( $piece['weight'] ) : null,
							'length'      => ! empty( $piece['length'] ) ? $this->normalizer->integer( $piece['length'] ) : null,
							'width'       => ! empty( $piece['width'] ) ? $this->normalizer->integer( $piece['width'] ) : null,
							'height'      => ! empty( $piece['height'] ) ? $this->normalizer->integer( $piece['height'] ) : null,
							'nonStandard' => ! empty( $piece['nonStandard'] ) ? $this->normalizer->bool( $piece['nonStandard'] ) : null,
						);
					}
				}
				$data['piece'] = $pieces;
			}

			$data['content']        = ! empty( $data['content'] ) ? sanitize_text_field( $data['content'] ) : '';
			$data['comment']        = ! empty( $data['comment'] ) ? sanitize_text_field( $data['comment'] ) : '';
			$data['reference']      = ! empty( $data['reference'] ) ? sanitize_text_field( $data['reference'] ) : '';
			$data['costs_center']   = ! empty( $data['costs_center'] ) ? sanitize_text_field( $data['costs_center'] ) : '';
			$data['label_exp_date'] = ! empty( $data['label_exp_date'] ) ? $this->normalizer->date( $data['label_exp_date'] ) : null;
			$data['book_courier']   = ! empty( $data['book_courier'] ) ? $this->normalizer->bool( $data['book_courier'] ) : false;

			// Validate.
			$is_error = false;
			if ( empty( $data['content'] ) ) {
				Notice::get_instance()->add( 'error', __( 'Field "Content" is required.', 'woocommerce-dhl24' ) );
				$is_error = true;
			}
			if ( true === $is_error ) {
				wp_send_json( Notice::get_instance()->get_notices(), \WP_Http::UNPROCESSABLE_ENTITY );
			}

			$create_shipment = new CreateShipmentContext( $shipping_method, $order_id, $data );

			try {
				$shipment = $create_shipment->create();
			} catch ( ExceptionInterface $e ) {
				wp_send_json( array( esc_attr( $e->getMessage() ) ), \WP_Http::UNPROCESSABLE_ENTITY );
			}

			$this->send_label( $shipment['fileName'], $shipment['mimeType'], $shipment['fileData'], $order_id );
		}

		return false;
	}

	/**
	 * Shipment confirmation
	 *
	 * @param  string|string[] $data Unslashed $data.
	 * @return bool
	 */
	public function shipment_confirmation( $data ): bool {
		if ( ! empty( $data['_nonce'] ) && wp_verify_nonce( sanitize_text_field( $data['_nonce'] ), 'ajax_shipment_confirmation' ) ) {
			$date = ! empty( $data['date'] ) ? $this->normalizer->date( $data['date'] ) : null;
			if ( null === $date ) {
				$date = new \DateTime();
			}
			$type = ! empty( $data['type'] ) ? sanitize_text_field( $data['type'] ) : '';
			$type = strtoupper( $type );
			if ( ! in_array( $type, array( 'ALL', 'EX', 'DR', '2EUROPE', 'PR' ), true ) ) {
				$type = 'ALL';
			}

			$shipments_repository = new ShipmentsRepository();

			try {
				$pnp = $shipments_repository->get_pnp( $date, $type );
			} catch ( ExceptionInterface $e ) {
				wp_send_json( array( esc_attr( $e->getMessage() ) ), \WP_Http::UNPROCESSABLE_ENTITY );
			}

			$this->send_label( $pnp->getFileName(), $pnp->getFileMimeType(), $pnp->getFileData() );
		}
		return false;
	}

	/**
	 * Shipment template
	 *
	 * @param  string|string[] $data Unslashed $data.
	 * @return bool
	 */
	public function shipment_template( $data ): bool {
		if ( ! empty( $data['_nonce'] ) && wp_verify_nonce( sanitize_text_field( $data['_nonce'] ), 'ajax_shipment_template' ) ) {
			$data['id']                             = ! empty( $data['id'] ) ? $this->normalizer->integer( $data['id'] ) : 0;
			$data['name']                           = ! empty( $data['name'] ) ? sanitize_text_field( $data['name'] ) : '';
			$data['piece_type']                     = ! empty( $data['piece_type'] ) ? sanitize_text_field( $data['piece_type'] ) : '';
			$data['piece_quantity']                 = ! empty( $data['piece_quantity'] ) ? $this->normalizer->float( $data['piece_quantity'] ) : '';
			$data['piece_weight']                   = ! empty( $data['piece_weight'] ) ? $this->normalizer->float( $data['piece_weight'] ) : '';
			$data['piece_width']                    = ! empty( $data['piece_width'] ) ? $this->normalizer->float( $data['piece_width'] ) : '';
			$data['piece_height']                   = ! empty( $data['piece_height'] ) ? $this->normalizer->float( $data['piece_height'] ) : '';
			$data['piece_length']                   = ! empty( $data['piece_length'] ) ? $this->normalizer->float( $data['piece_length'] ) : '';
			$data['piece_nonStandard']              = ! empty( $data['piece_nonStandard'] ) ? $this->normalizer->bool( $data['piece_nonStandard'] ) : '';
			$data['piece_euroReturn']               = ! empty( $data['piece_euroReturn'] ) ? $this->normalizer->bool( $data['piece_euroReturn'] ) : '';
			$data['content']                        = ! empty( $data['content'] ) ? sanitize_text_field( $data['content'] ) : '';
			$data['comment']                        = ! empty( $data['comment'] ) ? sanitize_text_field( $data['comment'] ) : '';
			$data['reference']                      = ! empty( $data['reference'] ) ? sanitize_text_field( $data['reference'] ) : '';
			$data['costs_center']                   = ! empty( $data['costs_center'] ) ? sanitize_text_field( $data['costs_center'] ) : '';
			$data['service_insurance']              = ! empty( $data['service_insurance'] ) ? $this->normalizer->bool( $data['service_insurance'] ) : '';
			$data['service_insurance_value']        = ! empty( $data['service_insurance_value'] ) ? $this->normalizer->float( $data['service_insurance_value'] ) : '';
			$data['service_predeliveryInformation'] = ! empty( $data['service_predeliveryInformation'] ) ? $this->normalizer->bool( $data['service_predeliveryInformation'] ) : '';

			// Validate.
			$is_error = false;
			if ( 0 > $data['id'] ) {
				$data['id'] = 0;
			}
			if ( empty( $data['name'] ) ) {
				Notice::get_instance()->add( 'error', __( 'Field "Name" is required.', 'woocommerce-dhl24' ) );
				$is_error = true;
			}
			if ( true === $is_error ) {
				wp_send_json( Notice::get_instance()->get_notices(), \WP_Http::UNPROCESSABLE_ENTITY );
			}
			Template::get_instance()->save( $data );

			wp_send_json( array(), \WP_Http::OK );
		}
		return false;
	}

	/**
	 * Shipment template delete
	 *
	 * @param  string|string[] $data Unslashed $data.
	 * @return bool
	 */
	public function shipment_template_delete( $data ): bool {
		$id = ! empty( $data['id'] ) ? $this->normalizer->integer( $data['id'] ) : 0;
		if ( ! empty( $data['_nonce'] ) && wp_verify_nonce( sanitize_text_field( $data['_nonce'] ), 'ajax_shipment_template_delete_' . $id ) ) {
			Template::get_instance()->delete( $id );

			wp_send_json( array(), \WP_Http::NO_CONTENT );
		}
		return false;
	}

	/**
	 * Send label
	 *
	 * @param  string|null $file_name File name.
	 * @param  string|null $mime_type Mime type.
	 * @param  string|null $file_data File data.
	 * @param  int|null    $order_id  Order ID.
	 * @return void
	 */
	private function send_label( $file_name, $mime_type, $file_data, $order_id = null ) {
		wp_send_json(
			array(
				'fileName' => $file_name,
				'mimeType' => $mime_type,
				'data'     => $file_data,
				'id'       => $order_id,
			),
			\WP_Http::OK
		);
	}
}
