<?php
/**
 * Class CreateShipmentReturnWebAPI.
 *
 * @package ILabs\WoocommerceDHL24
 */

namespace ILabs\WoocommerceDHL24\Helper;

use ILabs\WoocommerceDHL24\Exception\HelperException;
use ILabs\WoocommerceDHL24\Exception\ExceptionInterface;
use ILabs\WoocommerceDHL24\Repository\ShipmentRepository;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\Billing;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ReturnShip;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ReturnPackage;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ReturnService;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ShipperAddress;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\PreavisoContact;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ReturnAddressat;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ReturnShipmentInfo;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ReturnShipmentTime;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ReceiverAddressLabel;

/**
 * CreateShipmentReturnWebAPI
 */
class CreateShipmentReturnWebAPI implements CreateShipmentInterface {


	/**
	 * Order ID
	 *
	 * @var int
	 */
	private $order_id;

	/**
	 * Posted data
	 *
	 * @var array
	 */
	private $data;

	/**
	 * Formatter
	 *
	 * @var Formatter
	 */
	protected $formatter;

	/**
	 * Construct
	 *
	 * @param  int   $order_id Order ID.
	 * @param  array $data Posted data.
	 * @return void
	 */
	public function __construct( int $order_id, array $data ) {
		$this->order_id = $order_id;
		$this->data     = $data;

		$this->formatter = new Formatter();
	}

	/**
	 * Create shipment
	 *
	 * @return array
	 * @throws \HelperException If error.
	 */
	public function create(): array {
		$ship = new ReturnShip();

		$receiver = new ReturnAddressat();
		$receiver->setPreaviso( $this->get_preaviso() );
		$receiver->setContact( $this->get_contact() );
		$receiver->setAddress( $this->get_receiver_address() );
		$ship->setReceiver( $receiver );

		$ship->setShipper( $this->get_shipper() );

		$pieces = array();
		if ( isset( $this->data['piece'] ) && is_array( $this->data['piece'] ) ) {
			foreach ( $this->data['piece'] as $item ) {
				$piece = new ReturnPackage();
				$piece->setType( $item['type'] );
				$piece->setQuantity( $item['quantity'] );
				$piece->setWeight( $item['weight'] );
				$piece->setLength( $item['length'] );
				$piece->setWidth( $item['width'] );
				$piece->setHeight( $item['height'] );
				$piece->setNonStandard( $item['nonStandard'] );
				$pieces[] = $piece;
			}
		} else {
			$pieces[] = $this->get_piece();
		}

		if ( isset( $this->data['content'] ) ) {
			$content = $this->data['content'];
		} else {
			$content = Option::get_instance()->get_value( 'content' );
		}
		if ( isset( $this->data['comment'] ) ) {
			$comment = $this->data['comment'];
		} else {
			$comment = Option::get_instance()->get_value( 'comment' );
		}
		$reference = null;
		if ( isset( $this->data['reference'] ) ) {
			$reference = $this->data['reference'];
		}
		$primary_waybill_number = null;
		if ( isset( $this->data['primary_waybill_number'] ) ) {
			$primary_waybill_number = $this->data['primary_waybill_number'];
		}

		$shipment_data = array(
			'ship'                   => $ship,
			'shipment_info'          => $this->get_shipment_info(),
			'pieces'                 => $pieces,
			'content'                => $content,
			'comment'                => $comment,
			'reference'              => $reference,
			'primary_waybill_number' => $primary_waybill_number,
		);

		$service_point_repository = new ShipmentRepository();
		try {
			$return_shipments = $service_point_repository->create_shipment_return( $shipment_data );
		} catch ( ExceptionInterface $e ) {
			throw new HelperException( $e, $e->getMessage() );
		}

		$shipments        = array();
		$dispatch_numbers = array();
		$result           = array(
			'return_shipment_id'     => '',
			'return_dispatch_number' => '',
			'mimeType'               => null,
			'fileData'               => null,
		);
		foreach ( $return_shipments->getItems() as $shipment ) {
			$shipments[]                      = $shipment->getShipmentNotificationNumber();
			$dispatch_numbers[]               = $shipment->getDispatchNotificationNumber();
			$result['return_shipment_id']     = $shipment->getShipmentNotificationNumber();
			$result['return_dispatch_number'] = $shipment->getDispatchNotificationNumber();
			$label                            = $shipment->getLabel();
			$result['mimeType']               = $label->getLabelFormat();
			$result['fileData']               = $label->getLabelContent();
		}

		update_post_meta( $this->order_id, 'dhl24_return_shipments', implode( ',', $shipments ) );
		update_post_meta( $this->order_id, 'dhl24_return_dispatch_numbers', implode( ',', $dispatch_numbers ) );

		return $result;
	}

	/**
	 * Get company address
	 *
	 * @return ShipperAddress
	 */
	private function get_receiver_address(): ShipperAddress {
		$data = Option::get_instance()->get_data();

		$address = new ShipperAddress();
		$address->setCountry( $data['return_country'] );
		$address->setName( $data['return_name'] );
		$address->setPostalCode( $this->formatter->post_code( $data['return_postal_code'], $data['return_country'] ) );
		$address->setCity( $data['return_city'] );
		$address->setStreet( $data['return_street'] );
		$address->setHouseNumber( $data['return_house_nmber'] );
		$address->setApartmentNumber( $data['return_apartment_number'] );

		return $address;
	}

	/**
	 * Get company contact
	 *
	 * @return PreavisoContact
	 */
	private function get_contact(): PreavisoContact {
		$data = Option::get_instance()->get_data();

		$contact = new PreavisoContact();
		$contact->setPersonName( $data['return_contact_person'] );
		$contact->setPhoneNumber( $data['return_contact_phone'] );
		$contact->setEmailAddress( $data['return_contact_email'] );

		return $contact;
	}

	/**
	 * Get company preaviso
	 *
	 * @return PreavisoContact
	 */
	private function get_preaviso(): PreavisoContact {
		$data = Option::get_instance()->get_data();

		$preaviso = new PreavisoContact();
		$preaviso->setPersonName( $data['return_contact_person'] );
		$preaviso->setPhoneNumber( $data['return_contact_phone'] );
		$preaviso->setEmailAddress( $data['return_contact_email'] );

		return $preaviso;
	}

	/**
	 * Get piece
	 *
	 * @return ReturnPackage
	 */
	private function get_piece(): ReturnPackage {
		$data = Option::get_instance()->get_data();

		$piece = new ReturnPackage();
		$piece->setType( $data['piece_type'] );
		$piece->setWidth( $data['piece_width'] );
		$piece->setHeight( $data['piece_height'] );
		$piece->setLength( $data['piece_length'] );
		$piece->setWeight( $data['piece_weight'] );
		$piece->setQuantity( $data['piece_quantity'] );

		return $piece;
	}

	/**
	 * Get piece
	 *
	 * @return ReturnShipmentInfo
	 */
	private function get_shipment_info(): ReturnShipmentInfo {
		$default_data = Option::get_instance()->get_data();

		if ( ! isset( $this->data['costs_center'] ) ) {
			$this->data['costs_center'] = $default_data['costs_center'];
		}
		if ( ! isset( $this->data['label_exp_date'] ) ) {
			$services                     = new Services();
			$this->data['label_exp_date'] = $services->get_label_exp_date();
		}
		if ( ! isset( $this->data['book_courier'] ) ) {
			$this->data['book_courier'] = false;
		}
		/**
		 * Jeśli produkt inny niż krajowy to bookCourier zawsze false,
		 * nawet jak zaznaczony checbox "możliwość nadania przesyłki zwrotnej kurierem".
		 */
		if ( 'ZK' !== $this->data['shipping_method'] ) {
			$this->data['book_courier'] = false;
		}

		$shipment_info = new ReturnShipmentInfo();
		$shipment_info->setServiceType( $this->data['shipping_method'] );

		$billing = new Billing();
		$billing->setShippingPaymentType( 'RECEIVER' );
		$billing->setBillingAccountNumber( $default_data['sap'] );
		$billing->setPaymentType( 'BANK_TRANSFER' );
		$billing->setCostsCenter( $this->data['costs_center'] );
		$shipment_info->setBilling( $billing );

		$return_shipment_time = new ReturnShipmentTime();
		$return_shipment_time->setLabelExpDate( $this->data['label_exp_date'] );
		$shipment_info->setShipmentTime( $return_shipment_time );

		$shipment_info->setBookCourier( $this->data['book_courier'] );

		$shipment_info->setLabelType( $default_data['label_type'] );

		if ( isset( $this->data['service'] ) && is_array( $this->data['service'] ) ) {
			if ( true === $this->data['service']['insurance'] ) {
				$service = new ReturnService();
				$service->setServiceType( 'UBEZP' );
				$service->setServiceValue( $this->data['service']['insuranceValue'] );
				$shipment_info->addSpecialServices( $service );
			}
		}

		return $shipment_info;
	}

	/**
	 * Get shipper
	 *
	 * @return ReturnAddressat
	 */
	private function get_shipper(): ReturnAddressat {
		$return_addressat = new ReturnAddressat();

		if ( isset( $this->data['shipper'] ) && $this->data['shipper'] instanceof ReceiverAddressLabel ) {
			/**
			 * ReceiverAddressLabel
			 *
			 * @var ReceiverAddressLabel $shipper
			 */
			$shipper         = $this->data['shipper'];
			$shipper_address = new ShipperAddress();
			$shipper_address->setCountry( $shipper->getCountry() );
			$shipper_address->setName( $shipper->getName() );
			$shipper_address->setPostalCode( $this->formatter->post_code( $shipper->getPostalCode(), $shipper->getCountry() ) );
			$shipper_address->setCity( $shipper->getCity() );
			$shipper_address->setStreet( $shipper->getStreet() );
			$shipper_address->setHouseNumber( $shipper->getHouseNumber() );
			$shipper_address->setApartmentNumber( $shipper->getApartmentNumber() );
			$return_addressat->setAddress( $shipper_address );

			$preaviso = new PreavisoContact();
			$preaviso->setPersonName( $shipper->getPreavisoPerson() );
			$preaviso->setPhoneNumber( $shipper->getPreavisoPhone() );
			$preaviso->setEmailAddress( $shipper->getPreavisoEmail() );
			$return_addressat->setPreaviso( $preaviso );

			$contact = new PreavisoContact();
			$contact->setPersonName( $shipper->getContactPerson() );
			$contact->setPhoneNumber( $shipper->getContactPhone() );
			$contact->setEmailAddress( $shipper->getContactEmail() );
			$return_addressat->setcontact( $contact );
		} else {
			$name = $this->data['shipping_company'];
			if ( empty( $name ) ) {
				$name = $this->data['shipping_first_name'] . ' ' . $this->data['shipping_last_name'];
			}
			$street = $this->formatter->street( $this->data['shipping_address_1'] );

			$shipper_address = new ShipperAddress();
			$shipper_address->setCountry( $this->data['shipping_country'] );
			$shipper_address->setName( $name );
			$shipper_address->setPostalCode( $this->formatter->post_code( $this->data['shipping_postcode'], $this->data['shipping_country'] ) );
			$shipper_address->setCity( $this->data['shipping_city'] );
			$shipper_address->setStreet( $street['street'] );
			$shipper_address->setHouseNumber( $street['house_number'] );
			$shipper_address->setApartmentNumber( $this->data['shipping_address_2'] );
			$return_addressat->setAddress( $shipper_address );

			$preaviso = new PreavisoContact();
			$preaviso->setPersonName( $this->data['shipping_first_name'] . ' ' . $this->data['shipping_last_name'] );
			$preaviso->setPhoneNumber( $this->data['billing_phone'] );
			$preaviso->setEmailAddress( $this->data['billing_email'] );
			$return_addressat->setPreaviso( $preaviso );
			$return_addressat->setContact( $preaviso );
		}

		return $return_addressat;
	}
}
