<?php
/**
 * Class CreateShipmentServicePoint.
 *
 * @package ILabs\WoocommerceDHL24
 */

namespace ILabs\WoocommerceDHL24\Helper;

use ILabs\WoocommerceDHL24\Exception\HelperException;
use ILabs\WoocommerceDHL24\Exception\ExceptionInterface;
use ILabs\WoocommerceDHL24\Repository\ServicePointRepository;
use ILabs\WoocommerceDHL24\DHL24Client\ServicePoint\Type\Ship;
use ILabs\WoocommerceDHL24\DHL24Client\ServicePoint\Type\Piece;
use ILabs\WoocommerceDHL24\DHL24Client\ServicePoint\Type\Address;
use ILabs\WoocommerceDHL24\DHL24Client\ServicePoint\Type\Billing;
use ILabs\WoocommerceDHL24\DHL24Client\ServicePoint\Type\Contact;
use ILabs\WoocommerceDHL24\DHL24Client\ServicePoint\Type\Service;
use ILabs\WoocommerceDHL24\DHL24Client\ServicePoint\Type\Preaviso;
use ILabs\WoocommerceDHL24\DHL24Client\ServicePoint\Type\Receiver;
use ILabs\WoocommerceDHL24\DHL24Client\ServicePoint\Type\FullAddress;
use ILabs\WoocommerceDHL24\DHL24Client\ServicePoint\Type\ShipmentInfo;

/**
 * CreateShipmentServicePoint
 */
class CreateShipmentServicePoint implements CreateShipmentInterface {

	/**
	 * Order ID
	 *
	 * @var int
	 */
	private $order_id;

	/**
	 * Posted data
	 *
	 * @var array
	 */
	private $data;

	/**
	 * Formatter
	 *
	 * @var Formatter
	 */
	protected $formatter;

	/**
	 * Construct
	 *
	 * @param  int   $order_id Order ID.
	 * @param  array $data Posted data.
	 * @return void
	 */
	public function __construct( int $order_id, array $data ) {
		$this->order_id = $order_id;
		$this->data     = $data;

		$this->formatter = new Formatter();
	}

	/**
	 * Create shipment
	 *
	 * @return array
	 * @throws \HelperException If error.
	 */
	public function create(): array {
		if ( empty( $this->data['dhl24_sap'] ) ) {
			throw new HelperException( null, __( 'Select DHL POP.', 'woocommerce-dhl24' ) );
		}
		// TODO: SAP validation.
		$name = $this->data['shipping_company'];
		if ( empty( $name ) ) {
			$name = $this->data['shipping_first_name'] . ' ' . $this->data['shipping_last_name'];
		}
		$street = $this->formatter->street( $this->data['shipping_address_1'] );

		$ship = new Ship();

		$shipper = new FullAddress();
		$shipper->setAddress( $this->get_address() );
		$shipper->setContact( $this->get_contact() );
		$shipper->setPreaviso( $this->get_preaviso() );
		$ship->setShipper( $shipper );

		$receiver = new Receiver();

		$receiver_address = new Address();
		$receiver_address->setName( $name );
		$receiver_address->setPostcode( $this->formatter->post_code( $this->data['shipping_postcode'], $this->data['shipping_country'] ) );
		$receiver_address->setCity( $this->data['shipping_city'] );
		$receiver_address->setStreet( $street['street'] );
		$receiver_address->setHouseNumber( $street['house_number'] );
		$receiver_address->setApartmentNumber( $this->data['shipping_address_2'] );
		$receiver->setAddress( $receiver_address );

		$contact = new Contact();
		$contact->setPersonName( $this->data['shipping_first_name'] . ' ' . $this->data['shipping_last_name'] );
		$contact->setPhoneNumber( $this->data['billing_phone'] );
		$contact->setEmailAddress( $this->data['billing_email'] );
		$receiver->setContact( $contact );

		$preaviso = new Preaviso();
		$preaviso->setPersonName( $this->data['shipping_first_name'] . ' ' . $this->data['shipping_last_name'] );
		$preaviso->setPhoneNumber( $this->data['billing_phone'] );
		$preaviso->setEmailAddress( $this->data['billing_email'] );
		$receiver->setPreaviso( $preaviso );

		$ship->setServicePointAccountNumber( $this->data['dhl24_sap'] );

		$ship->setReceiver( $receiver );

		if ( isset( $this->data['content'] ) ) {
			$content = $this->data['content'];
		} else {
			$content = Option::get_instance()->get_value( 'content' );
		}
		if ( isset( $this->data['comment'] ) ) {
			$comment = $this->data['comment'];
		} else {
			$comment = Option::get_instance()->get_value( 'comment' );
		}
		$reference = null;
		if ( isset( $this->data['reference'] ) ) {
			$reference = $this->data['reference'];
		}

		$pieces = array();
		if ( isset( $this->data['piece'] ) && is_array( $this->data['piece'] ) ) {
			foreach ( $this->data['piece'] as $item ) {
				$piece = new Piece();
				$piece->setType( $item['type'] );
				$piece->setWidth( $item['width'] );
				$piece->setHeight( $item['height'] );
				$piece->setLength( $item['length'] );
				$piece->setWeight( $item['weight'] );
				$piece->setQuantity( $item['quantity'] );
				$pieces[] = $piece;
			}
		} else {
			$pieces[] = $this->get_piece();
		}

		$shipment_data = array(
			'ship'          => $ship,
			'shipment_info' => $this->get_shipment_info(),
			'pieces'        => $pieces,
			'content'       => $content,
			'comment'       => $comment,
			'reference'     => $reference,
		);

		$service_point_repository = new ServicePointRepository();
		try {
			$shipment = $service_point_repository->create_shipment( $shipment_data );
		} catch ( ExceptionInterface $e ) {
			throw new HelperException( $e, $e->getMessage() );
		}
		$shipment_id     = $shipment->getShipmentNumber();
		$dispatch_number = $shipment->getDispatchNumber();
		$label           = $shipment->getLabel();

		if ( ! empty( $this->order_id ) ) {
			update_post_meta( $this->order_id, 'dhl24_shipment_id', $shipment_id );
			update_post_meta( $this->order_id, 'dhl24_dispatch_number', $dispatch_number );
		}

		return array(
			'shipment_id'     => $shipment_id,
			'dispatch_number' => $dispatch_number,
			'mimeType'        => $label->getLabelFormat(),
			'fileData'        => $label->getLabelContent(),
		);
	}

	/**
	 * Get company address
	 *
	 * @return Address
	 */
	private function get_address(): Address {
		$data = Option::get_instance()->get_data();

		$address = new Address();
		$address->setName( $data['name'] );
		$address->setPostcode( $this->formatter->post_code( $data['postal_code'], $data['country'] ) );
		$address->setCity( $data['city'] );
		$address->setStreet( $data['street'] );
		$address->setHouseNumber( $data['house_nmber'] );
		$address->setApartmentNumber( $data['apartment_number'] );

		return $address;
	}

	/**
	 * Get company contact
	 *
	 * @return Contact
	 */
	private function get_contact(): Contact {
		$data = Option::get_instance()->get_data();

		$contact = new Contact();
		$contact->setPersonName( $data['contact_person'] );
		$contact->setPhoneNumber( $data['contact_phone'] );
		$contact->setEmailAddress( $data['contact_email'] );

		return $contact;
	}

	/**
	 * Get company preaviso
	 *
	 * @return Preaviso
	 */
	private function get_preaviso(): Preaviso {
		$data = Option::get_instance()->get_data();

		$preaviso = new Preaviso();
		$preaviso->setPersonName( $data['contact_person'] );
		$preaviso->setPhoneNumber( $data['contact_phone'] );
		$preaviso->setEmailAddress( $data['contact_email'] );

		return $preaviso;
	}

	/**
	 * Get piece
	 *
	 * @return Piece
	 */
	private function get_piece(): Piece {
		$data = Option::get_instance()->get_data();

		$piece = new Piece();
		$piece->setType( $data['piece_type'] );
		$piece->setWidth( $data['piece_width'] );
		$piece->setHeight( $data['piece_height'] );
		$piece->setLength( $data['piece_length'] );
		$piece->setWeight( $data['piece_weight'] );
		$piece->setQuantity( $data['piece_quantity'] );

		return $piece;
	}

	/**
	 * Get ShipmentInfo
	 *
	 * @return ShipmentInfo
	 */
	private function get_shipment_info(): ShipmentInfo {
		$default_data = Option::get_instance()->get_data();

		if ( ! isset( $this->data['costs_center'] ) ) {
			$this->data['costs_center'] = $default_data['costs_center'];
		}

		$shipment_info = new ShipmentInfo();
		if ( 'yes' === $default_data['auto_order_courier'] ) {
			$shipment_info->setDropOffType( 'REQUEST_COURIER' );
		} else {
			$shipment_info->setDropOffType( 'REGULAR_PICKUP' );
		}
		$shipment_info->setServiceType( $this->data['shipping_method'] );

		$billing = new Billing();
		$billing->setShippingPaymentType( 'SHIPPER' );
		$billing->setBillingAccountNumber( $default_data['sap'] );
		$billing->setPaymentType( 'BANK_TRANSFER' );
		$billing->setCostsCenter( $this->data['costs_center'] );
		$shipment_info->setBilling( $billing );

		$services = new Services();
		$shipment_info->setShipmentDate( $services->get_pickup_day( $default_data['postal_code'] ) );
		$shipment_info->setShipmentStartHour( $default_data['shipment_start_hour'] );
		$shipment_info->setShipmentEndHour( $default_data['shipment_end_hour'] );
		$shipment_info->setLabelType( $default_data['label_type'] );

		if ( isset( $this->data['service'] ) && is_array( $this->data['service'] ) ) {
			if ( true === $this->data['service']['insurance'] ) {
				$service = new Service();
				$service->setServiceType( 'UBEZP' );
				$service->setServiceValue( $this->data['service']['insuranceValue'] );
				$shipment_info->addSpecialServices( $service );
			}
			if ( true === $this->data['service']['collectOnDelivery'] ) {
				$service = new Service();
				$service->setServiceType( 'COD' );
				$service->setServiceValue( $this->data['service']['collectOnDeliveryValue'] );
				$service->setCollectOnDeliveryForm( 'BANK_TRANSFER' );
				$shipment_info->addSpecialServices( $service );
			}
		}

		if ( isset( $this->data['shipment_date'] ) ) {
			$shipment_info->setShipmentDate( $this->data['shipment_date'] );
		}

		return $shipment_info;
	}
}
