<?php
/**
 * Class Services.
 *
 * @package ILabs\WoocommerceDHL24
 */

namespace ILabs\WoocommerceDHL24\Helper;

use ILabs\WoocommerceDHL24\Helper\Option;
use ILabs\WoocommerceDHL24\Repository\ServiceRepository;

/**
 * Services
 */
class Services {

	/**
	 * Service repository
	 *
	 * @var ServiceRepository
	 */
	private $service_repository;

	/**
	 * Construct
	 *
	 * @return void
	 */
	public function __construct() {
		$this->service_repository = new ServiceRepository();
	}

	/**
	 * Get pickup days.
	 *
	 * @param  string $post_code Postal code.
	 * @param  bool   $is_dr_pickup Is drPickup.
	 * @param  string $order_courier Auto order courier.
	 * @param  string $service_type Service type.
	 * @return array|\DateTime[]
	 */
	public function get_pickup_days( string $post_code, bool $is_dr_pickup = false, string $order_courier = 'yes', string $service_type = '' ): array {
		$pickup_day = new \DateTime( 'yesterday' );
		$result     = array();
		$i          = 0;
		$max        = 3;

		if ( 'PK' === $service_type ) {
			$pickup_day->add( new \DateInterval( 'P1D' ) );
		}

		if ( 'yes' !== $order_courier ) {
			while ( $i < $max ) {
				$pickup_day->add( new \DateInterval( 'P1D' ) );
				if ( '7' !== $pickup_day->format( 'N' ) ) {
					$result[] = clone $pickup_day;
					$i++;
				}
			}

			return $result;
		}

		while ( $i < $max ) {
			$pickup_day->add( new \DateInterval( 'P1D' ) );
			try {
				$postal_code_services = $this->service_repository->fetch_service( $post_code, $pickup_day );
			} catch ( \Exception $e ) {
				$i++;
				continue;
			}
			if ( true === $is_dr_pickup ) {
				$from = \DateTime::createFromFormat( 'H:i', $postal_code_services->getDrPickupFrom() );
				$to   = \DateTime::createFromFormat( 'H:i', $postal_code_services->getDrPickupTo() );
			} else {
				$from = \DateTime::createFromFormat( 'H:i', $postal_code_services->getExPickupFrom() );
				$to   = \DateTime::createFromFormat( 'H:i', $postal_code_services->getExPickupTo() );
			}
			if ( false !== $from && false !== $to ) {
				$result[] = clone $pickup_day;
				$i++;
			}
		}

		return $result;
	}

	/**
	 * Get available hours.
	 *
	 * @param  string    $post_code Postal code.
	 * @param  \DateTime $pickup_date Date for which the availability of services is checked.
	 * @param  bool      $is_dr_pickup Is drPickup.
	 * @return array
	 */
	public function get_available_hours( string $post_code, \DateTime $pickup_date, bool $is_dr_pickup = false ): array {
		$result = array(
			'from'  => '',
			'to'    => '',
			'hours' => array(),
		);

		try {
			$postal_code_services = $this->service_repository->fetch_service( $post_code, $pickup_date );
		} catch ( \Exception $e ) {
			return $result;
		}

		if ( true === $is_dr_pickup ) {
			$result['from'] = \DateTime::createFromFormat( 'H:i', $postal_code_services->getDrPickupFrom() );
			$result['to']   = \DateTime::createFromFormat( 'H:i', $postal_code_services->getDrPickupTo() );
		} else {
			$result['from'] = \DateTime::createFromFormat( 'H:i', $postal_code_services->getExPickupFrom() );
			$result['to']   = \DateTime::createFromFormat( 'H:i', $postal_code_services->getExPickupTo() );
		}

		if ( ! empty( $result['from'] ) && ! empty( $result['to'] ) && $result['to'] > $result['from'] ) {
			$hour = clone $result['from'];
			while ( $hour <= $result['to'] ) {
				$interval = $result['to']->diff( $hour );
				if ( '00:59' > $interval->format( '%H:%I' ) ) {
					$hour = clone $result['to'];
				}
				$result['hours'][] = $hour->format( 'H:i' );
				$hour->add( new \DateInterval( 'PT30M' ) );
			}
		}

		return $result;
	}

	/**
	 * Get fist pickup day
	 *
	 * @param  string $post_code Postal code.
	 * @param  bool   $is_dr_pickup Is drPickup.
	 * @return \DateTime|null
	 */
	public function get_pickup_day( string $post_code, bool $is_dr_pickup = false ) {
		if ( 'yes' !== Option::get_instance()->get_value( 'auto_order_courier' ) ) {
			return new \DateTime();
		}
		$pickup_day = new \DateTime( 'yesterday' );
		$i          = 0;
		$max        = 7;
		while ( $i < $max ) {
			$pickup_day->add( new \DateInterval( 'P1D' ) );
			$i++;

			try {
				$postal_code_services = $this->service_repository->fetch_service( $post_code, $pickup_day );
			} catch ( \Exception $e ) {
				return null;
			}
			if ( true === $is_dr_pickup ) {
				$from = \DateTime::createFromFormat( 'H:i', $postal_code_services->getDrPickupFrom() );
				$to   = \DateTime::createFromFormat( 'H:i', $postal_code_services->getDrPickupTo() );
			} else {
				$from = \DateTime::createFromFormat( 'H:i', $postal_code_services->getExPickupFrom() );
				$to   = \DateTime::createFromFormat( 'H:i', $postal_code_services->getExPickupTo() );
			}
			if ( false !== $from && false !== $to ) {
				return $pickup_day;
			}
		}

		return null;
	}

	/**
	 * Is service available
	 *
	 * @param  string $service_type Service type.
	 * @param  string $country Country.
	 * @param  string $post_code Postal code.
	 * @return bool
	 */
	public function is_service_available( string $service_type, string $country, string $post_code ): bool {
		return true;

		if ( in_array( $service_type, array( 'EK', 'PI' ), true ) && 'PL' === $country ) {
			return false;
		} elseif ( in_array( $service_type, array( '09', '12' ), true ) && 'PL' === $country ) {
			try {
				$pickup_day           = new \DateTime();
				$postal_code_services = $this->service_repository->fetch_service( $post_code, $pickup_day );
			} catch ( \Exception $e ) {
				return false;
			}
			if ( '09' === $service_type ) {
				return $postal_code_services->getDomesticExpress9();
			}
			if ( '12' === $service_type ) {
				return $postal_code_services->getDomesticExpress12();
			}
		}
		return true;
	}

	/**
	 * Get label expiry date
	 *
	 * @return \DateTime
	 */
	public function get_label_exp_date() {
		$label_exp = (int) Option::get_instance()->get_value( 'return_label_exp' );
		if ( $label_exp >= 7 && $label_exp <= 183 ) {
			return new \DateTime( $label_exp . ' day' );
		}
		return new \DateTime( '14 day' );
	}
}
