<?php
/**
 * Class Shipment.
 *
 * @package ILabs\WoocommerceDHL24
 */

namespace ILabs\WoocommerceDHL24\Helper;

use ILabs\WoocommerceDHL24\Helper\Param;
use ILabs\WoocommerceDHL24\Exception\ExceptionInterface;
use ILabs\WoocommerceDHL24\Repository\ShipmentRepository;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\Shipments;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\LabelsDatas;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\TrackAndTrace;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\DeleteNotifications;
use WP_Query;

/**
 * Shipment
 */
class Shipment {

	/**
	 * Construct
	 *
	 * @return void
	 */
	public function __construct() {
	}

	/**
	 * Get new shipment page args
	 *
	 * @param int $order_id Order ID.
	 * @return array
	 */
	public function get_new_page_args( int $order_id ): array {
		$order = wc_get_order( $order_id );
		if ( false === $order ) {
			/* translators: %s: Order ID */
			Notice::get_instance()->add( 'error', sprintf( __( 'Order %s does not exist.', 'woocommerce-dhl24' ), $order_id ) );

			return array();
		}
		$shipment_id = get_post_meta( $order_id, 'dhl24_shipment_id', true );
		if ( ! empty( $shipment_id ) ) {
			/* translators: %s: Order ID */
			Notice::get_instance()->add( 'error', sprintf( __( 'Order %s has got created shipment.', 'woocommerce-dhl24' ), $order_id ) );

			return array();
		}

		$shipping_methods    = Param::get_instance()->get_value( 'shipping_methods' );
		$shipping_method_ids = array();
		foreach ( $order->get_shipping_methods() as $shipping_method ) {
			if ( ! in_array( $shipping_method->get_method_id(), $shipping_methods, true ) ) {
				Notice::get_instance()->add( 'error', 'Shipping method not allowed.' );

				return array();
			}
			$shipping_method_ids[] = $shipping_method->get_method_id();
		}
		$shipping_method_ids = array_unique( $shipping_method_ids );

		if ( empty( $shipping_method_ids ) ) {
			Notice::get_instance()->add( 'error', 'Shipping method has not been set.' );

			return array();
		}
		$shipping_method_id = $shipping_method_ids[0];
		$services           = new Services();
		$pickup_days        = $services->get_pickup_days( Option::get_instance()->get_value( 'postal_code' ), false, Option::get_instance()->get_value( 'auto_order_courie' ) );

		$insurance                 = false;
		$insurance_value           = '';
		$collect_on_delivery       = false;
		$collect_on_delivery_value = '';
		if ( 'cod' === $order->get_payment_method() ) {
			$international_shipping_methods = Param::get_instance()->get_value( 'international_shipping_methods' );
			if ( ! in_array( $shipping_method_id, $international_shipping_methods, true ) ) {
				$insurance       = true;
				$insurance_value = number_format( $order->get_total(), 2, ',', ' ' );
			}
			$collect_on_delivery       = true;
			$collect_on_delivery_value = number_format( $order->get_total(), 2, ',', ' ' );
		}

		return array(
			'_nonce'                    => wp_create_nonce( 'ajax_new_shipment' ),
			'order_id'                  => $order_id,
			'order'                     => $order,
			'shipping_method_id'        => $shipping_method_id,
			'pickup_days'               => $pickup_days,
			'insurance'                 => $insurance,
			'insurance_value'           => $insurance_value,
			'collect_on_delivery'       => $collect_on_delivery,
			'collect_on_delivery_value' => $collect_on_delivery_value,
		);
	}

	/**
	 * Get return shipment page args
	 *
	 * @param  string $shipment_id Shipment ID.
	 * @return array
	 */
	public function get_return_page_args( string $shipment_id ): array {
		$order_id = $this->find_order_id( 'dhl24_shipment_id', $shipment_id );
		$order    = wc_get_order( $order_id );

		$services       = new Services();
		$label_exp_date = $services->get_label_exp_date();

		if ( empty( $shipment_id ) ) {
			Notice::get_instance()->add( 'error', __( 'Invalid data.', 'woocommerce-dhl24' ) );
			return array();
		}
		$shipment_repository = new ShipmentRepository();

		try {
			$labels = $shipment_repository->fetch_label( $shipment_id );
		} catch ( ExceptionInterface $e ) {
			Notice::get_instance()->add( 'error', $e->getMessage() );
			$labels = new LabelsDatas();
		}
		if ( 0 === $labels->getItems()->count() ) {
			Notice::get_instance()->add( 'error', __( 'Invalid data.', 'woocommerce-dhl24' ) );
			return array();
		}

		if ( false === $order ) {
			return array(
				'_nonce'         => wp_create_nonce( 'ajax_return_shipment' ),
				'labels_data'    => $labels->getItems()->offsetGet( 0 ),
				'label_exp_date' => $label_exp_date->format( 'Y-m-d' ),
			);
		}

		$shipping_method_ids = array();
		foreach ( $order->get_shipping_methods() as $shipping_method ) {
			$shipping_method_ids[] = $shipping_method->get_method_id();
		}
		$shipping_method_ids = array_unique( $shipping_method_ids );

		if ( empty( $shipping_method_ids ) ) {
			Notice::get_instance()->add( 'error', __( 'Shipping method has not been set.', 'woocommerce-dhl24' ) );

			return array();
		}
		$shipping_method_id = $shipping_method_ids[0];
		$international_shipping_methods = Param::get_instance()->get_value( 'international_shipping_methods' );
		if ( in_array( $shipping_method_id, $international_shipping_methods, true ) ) {
			$shipping_method = 'dhl24_service_zc';
		} else {
			$shipping_method = 'dhl24_service_zk';
		}

		return array(
			'_nonce'             => wp_create_nonce( 'ajax_return_shipment' ),
			'order_id'           => $order_id,
			'order'              => $order,
			'shipping_method_id' => $shipping_method,
			'shipment_id'        => $shipment_id,
			'labels_data'        => $labels->getItems(),
			'label_exp_date'     => $label_exp_date->format( 'Y-m-d' ),
		);
	}

	/**
	 * Cancel courier booking
	 *
	 * @param  string $courier_order_id Courier order number.
	 * @return void
	 */
	public function cancel_courier_booking( string $courier_order_id ) {
		$shipment_repository = new ShipmentRepository();
		$is_deleted          = true;
		try {
			$delete_notifications = $shipment_repository->cancel_courier_booking( $courier_order_id );
		} catch ( ExceptionInterface $e ) {
			Notice::get_instance()->add( 'error', $e->getMessage() );
			$delete_notifications = new DeleteNotifications();
			$is_deleted           = false;
		}

		if ( true === $is_deleted ) {
			$order_id = $this->find_order_id( 'dhl24_dispatch_number', $courier_order_id );
			if ( ! empty( $order_id ) ) {
				update_post_meta( $order_id, 'dhl24_dispatch_number', null );
			}
			$order_id = $this->find_order_id( 'dhl24_return_dispatch_numbers', $courier_order_id );
			if ( ! empty( $order_id ) ) {
				$return_dispatch_numbers = (string) get_post_meta( $order_id, 'dhl24_return_dispatch_numbers', true );
				$return_dispatch_numbers = explode( ',', $return_dispatch_numbers );
				$dispatch_numbers        = array();
				foreach ( $return_dispatch_numbers as $k => $value ) {
					if ( $value == $courier_order_id ) {
						$dispatch_numbers[ $k ] = '';
					} else {
						$dispatch_numbers[ $k ] = $return_dispatch_numbers[ $k ];
					}
				}
				update_post_meta( $order_id, 'dhl24_return_dispatch_numbers', implode( ',', $dispatch_numbers ) );
			}
		}

		/**
		 * Loop
		 *
		 * @var DeleteNotification $item DeleteNotification.
		 */
		foreach ( $delete_notifications->getItems() as $key => $item ) {
			if ( true === $item->getResult() ) {
				Notice::get_instance()->add( 'success', __( 'The order has been cancelled', 'woocommerce-dhl24' ) );
			} else {
				Notice::get_instance()->add( 'error', $item->getError() );
			}
		}
	}

	/**
	 * Delete shipment
	 *
	 * @param  string $shipment_id Shipment number of the shipment to be deleted.
	 * @return void
	 */
	public function delete( string $shipment_id ) {
		if ( empty( $shipment_id ) ) {
			Notice::get_instance()->add( 'error', __( 'Invalid data.', 'woocommerce-dhl24' ) );
			return;
		}
		$courier_order_id    = ! empty( $_GET['co_id'] ) ? sanitize_text_field( wp_unslash( $_GET['co_id'] ) ) : ''; // phpcs:ignore WordPress.Security.NonceVerification
		$shipment_repository = new ShipmentRepository();
		try {
			$delete_shipment = $shipment_repository->delete( $shipment_id, $courier_order_id );
		} catch ( ExceptionInterface $e ) {
			Notice::get_instance()->add( 'error', $e->getMessage() );
			return;
		}

		$order_id = $this->find_order_id( 'dhl24_shipment_id', $shipment_id );
		if ( ! empty( $order_id ) ) {
			delete_post_meta( $order_id, 'dhl24_shipment_id', null );
			delete_post_meta( $order_id, 'dhl24_dispatch_number', null );
			$sendback = admin_url( 'post.php?post=' . $order_id . '&action=edit' );
			wp_redirect( $sendback );
			exit();
		}
		// $order_id = $this->find_order_id( 'dhl24_return_shipment_id', $shipment_id );
		// if ( ! empty( $order_id ) ) {
		// delete_post_meta( $order_id, 'dhl24_return_shipment_id', null );
		// delete_post_meta( $order_id, 'dhl24_return_dispatch_number', null );
		// $sendback = admin_url( 'post.php?post=' . $order_id . '&action=edit' );
		// wp_redirect( $sendback );
		// exit();
		// }
		$order_id = $this->find_order_id( 'dhl24_return_shipments', $shipment_id );
		if ( ! empty( $order_id ) ) {
			$return_shipments        = (string) get_post_meta( $order_id, 'dhl24_return_shipments', true );
			$return_shipments        = explode( ',', $return_shipments );
			$return_dispatch_numbers = (string) get_post_meta( $order_id, 'dhl24_return_dispatch_numbers', true );
			$return_dispatch_numbers = explode( ',', $return_dispatch_numbers );
			$shipments               = array();
			$dispatch_numbers        = array();
			foreach ( $return_shipments as $key => $value ) {
				if ( $value !== $shipment_id ) {
					$shipments[]        = $value;
					$dispatch_numbers[] = isset( $return_dispatch_numbers[ $key ] ) ? $return_dispatch_numbers[ $key ] : '';
				}
			}
			update_post_meta( $order_id, 'dhl24_return_shipments', implode( ',', $shipments ) );
			update_post_meta( $order_id, 'dhl24_return_dispatch_numbers', implode( ',', $dispatch_numbers ) );
			$sendback = admin_url( 'post.php?post=' . $order_id . '&action=edit' );
			wp_redirect( $sendback );
			exit();
		}

		if ( true === $delete_shipment->getResult() ) {
			Notice::get_instance()->add( 'success', __( 'The shipment has been deleted', 'woocommerce-dhl24' ) );
		} else {
			Notice::get_instance()->add( 'error', $delete_shipment->getError() );
		}
	}

	/**
	 * Get shipment page args
	 *
	 * @param  string $shipment_id Shipment number.
	 * @return array
	 */
	public function get_show_page_args( string $shipment_id ): array {
		$shipment_repository = new ShipmentRepository();

		try {
			$labels = $shipment_repository->fetch_label( $shipment_id );
		} catch ( ExceptionInterface $e ) {
			Notice::get_instance()->add( 'error', $e->getMessage() );
			$labels = new LabelsDatas();
		}
		if ( 0 === $labels->getItems()->count() ) {
			return array();
		}

		try {
			$tracking = $shipment_repository->fetch_tracking( $shipment_id );
		} catch ( ExceptionInterface $e ) {
			Notice::get_instance()->add( 'error', $e->getMessage() );
			$tracking = new TrackAndTrace();
		}

		$labels_data = $labels->getItems()->offsetGet( 0 );
		$pieces      = new Pieces();
		$pieces->add_piece_list( $labels_data->getPieceList() );

		$meta_key = 'dhl24_shipment_id';
		$service  = $labels_data->getService();
		if ( in_array( $service->getProduct(), array( 'ZK', 'ZC' ), true ) ) {
			$meta_key = 'dhl24_return_shipments';
		}

		return array(
			'shipment_id'  => $shipment_id,
			'labels_data'  => $labels_data,
			'tracking'     => $tracking,
			'piece_totals' => $pieces->get_totals(),
			'order_id'     => $this->find_order_id( $meta_key, $shipment_id ),
		);
	}

	/**
	 * Return shipment by waybill
	 *
	 * @param  string $shipment_id Shipment number of the shipment to be deleted.
	 * @return string|null
	 */
	public function return_shipment_by_waybill( string $shipment_id ) {
		$shipment_repository = new ShipmentRepository();
		$shipments           = null;
		try {
			$shipments = $shipment_repository->return_shipment( $shipment_id );
		} catch ( ExceptionInterface $e ) {
			Notice::get_instance()->add( 'error', $e->getMessage() );
		}

		$shipment_notification_number = null;
		if ( $shipments instanceof Shipments ) {
			foreach ( $shipments->getItems() as $shipment ) {
				$shipment_notification_number = $shipment->getShipmentNotificationNumber();
			}
		}

		return $shipment_notification_number;
	}

	/**
	 * Find order ID
	 *
	 * @param  string $meta_key Meta key.
	 * @param  string $meta_value Meta value.
	 * @return int
	 */
	public function find_order_id( string $meta_key, string $meta_value ):int {
		if ( empty( $meta_value ) ) {
			return 0;
		}
		$args     = array(
			'post_type'    => wc_get_order_types(),
			'post_status'  => array_keys( wc_get_order_statuses() ),
			'fields'       => 'ids',
			'meta_key'     => $meta_key,
			'meta_value'   => $meta_value,
			'meta_compare' => 'LIKE',
		);
		$wp_query = new WP_Query( $args );
		$result   = $wp_query->get_posts();
		if ( ! empty( $result ) ) {
			return (int) $result[0];
		}
		return 0;
	}
}
