<?php
/**
 * Class View.
 *
 * @package ILabs\WoocommerceDHL24
 */

namespace ILabs\WoocommerceDHL24\Helper;

use ILabs\WoocommerceDHL24\Helper\Option;
use ILabs\WoocommerceDHL24\Helper\Defaults;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\Address;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\Billing;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\LabelsData;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ShipperAddressLabel;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\Point as WebAPIPoint;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ReceiverAddressLabel;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\LabelDataServiceDefinition;
use ILabs\WoocommerceDHL24\DHL24Client\ServicePoint\Type\Point as ServicePointPoint;

/**
 * View
 */
class View {

	/**
	 * Labels
	 *
	 * @var \ArrayObject
	 */
	private $labels;

	/**
	 * Construct
	 *
	 * @return void
	 */
	public function __construct() {
		$this->labels = new \ArrayObject();
	}

	/**
	 * Escape address.
	 *
	 * @param Address|null $address Address.
	 * @return string
	 */
	public function esc_address( $address ): string {
		if ( ! $address instanceof Address ) {
			return '';
		}

		$xhtml = esc_html( $address->getName() )
			. '<br />'
			. esc_html( $address->getStreet() . ' ' . $address->getHouseNumber() )
			. ( $address->getApartmentNumber() ? '/' . esc_html( $address->getApartmentNumber() ) : '' )
			. '<br />'
			. esc_html( $address->getPostalCode() . ' ' . $address->getCity() );

		return $xhtml;
	}

	/**
	 * Escape service point address.
	 *
	 * @param WebAPIPoint|ServicePointPoint|null $point Service point.
	 * @return string
	 */
	public function esc_point_address( $point ): string {
		$xhtml = '';

		if ( ! $point instanceof WebAPIPoint && ! $point instanceof ServicePointPoint ) {
			return $xhtml;
		}
		$xhtml .= '<strong>'
			. esc_html( $point->getName() )
			. '</strong>';

		$address = $point->getAddress();
		if ( null !== $address ) {
			$xhtml .= '<br />'
			. esc_html( $address->getPostCode() . ' ' . $address->getCity() )
			. '<br />'
			. esc_html( $address->getStreet() . ' ' . $address->getHouseNumber() )
			. ( $address->getApartmentNumber() ? '/' . esc_html( $address->getApartmentNumber() ) : '' );
		}

		return $xhtml;
	}


	/**
	 * Get select days.
	 *
	 * @return array|\DateTime[]
	 */
	public function get_select_days(): array {
		$select_date = new \DateTime();
		$result      = array();

		$i   = 0;
		$max = 3;
		while ( $i < $max ) {
			$result[] = clone $select_date;
			$select_date->add( new \DateInterval( 'P1D' ) );
			$i++;
		}

		return $result;
	}

	/**
	 * Get open hours.
	 *
	 * @param  WebAPIPoint $point Service point.
	 * @return array|\DateTime[]
	 */
	public function get_open_hours( WebAPIPoint $point ): array {
		$open_hours = array();
		foreach ( $this->get_select_days() as $select_date ) {
			$day              = $select_date->format( 'D' );
			$get_open_method  = 'get' . $day . 'Open';
			$get_close_method = 'get' . $day . 'Close';
			if ( method_exists( $point, $get_open_method ) && ! empty( $point->$get_open_method() ) ) {
				$open_hours[ $select_date->format( 'Y-m-d' ) ] = $point->$get_open_method();

				if ( method_exists( $point, $get_close_method ) && ! empty( $point->$get_close_method() ) ) {
					$open_hours[ $select_date->format( 'Y-m-d' ) ] .= ' - ' . $point->$get_close_method();
				}
			}
		};
		return $open_hours;
	}

	/**
	 * Set labels
	 *
	 * @param  \ArrayObject $labels Array of LabelsData.
	 * @return void
	 */
	public function set_labels( \ArrayObject $labels ) {
		$this->labels = $labels;
	}

	/**
	 * Get courier order id.
	 *
	 * @param  int|null $shipment_id Shipment ID.
	 * @return string
	 */
	public function get_courier_order_id( $shipment_id ): string {
		$courier_order_id = '';
		/**
		 * Loop
		 *
		 * @var LabelsData $label LabelsData.
		 */
		foreach ( $this->labels as $label ) {
			if ( $label->getShipmentId() === $shipment_id ) {
				if ( null !== $label->getDispatchNotificationNumber() ) {
					$courier_order_id = $label->getDispatchNotificationNumber();
				}
				break;
			}
		}
		return $courier_order_id;
	}

	/**
	 * Get shipment date.
	 *
	 * @param  int|null $shipment_id Shipment ID.
	 * @return string
	 */
	public function get_shipment_date( $shipment_id ): string {
		$shipment_date = null;
		/**
		 * Loop
		 *
		 * @var LabelsData $label LabelsData.
		 */
		foreach ( $this->labels as $label ) {
			if ( $label->getShipmentId() === $shipment_id ) {
				if ( null !== $label->getShipmentTime() ) {
					$shipment_date = $label->getShipmentTime()->getShipmentDate();
				}
				break;
			}
			if ( null !== $label->getService() && in_array( $label->getService()->getProduct(), array( 'EK', 'PI' ) ) ) {
				$find = false;
				foreach ( $label->getPieceList() as $piece ) {
					if ( $piece->getBlpPieceId() === $shipment_id ) {
						$shipment_date = $label->getShipmentTime()->getShipmentDate();
						$find          = true;
						break;
					}
				}
				if ( true === $find ) {
					break;
				}
			};

		}
		if ( null === $shipment_date ) {
			$shipment_date = '';
		}
		return $shipment_date;
	}

	/**
	 * Get coutry name
	 *
	 * @param  string $country_code Country code.
	 * @return string
	 */
	public function get_coutry_name( string $country_code ): string {
		if ( isset( WC()->countries->countries[ $country_code ] ) ) {
			return WC()->countries->countries[ $country_code ];
		}

		return '';
	}

	/**
	 * Get parcel type name
	 *
	 * @param  string $parcel_type Parcel type.
	 * @return string
	 */
	public function get_parcel_type_name( string $parcel_type ): string {
		$parcel_types = Param::get_instance()->get_value( 'parcel_types' );
		if ( isset( $parcel_types[ $parcel_type ] ) ) {
			return $parcel_types[ $parcel_type ];
		}

		return $parcel_type;
	}

	/**
	 * Escape order number.
	 *
	 * @param  string $courier_order_id Courier order ID.
	 * @return string
	 */
	public function esc_order_number( string $courier_order_id ): string {
		if ( '' === $courier_order_id ) {
			return esc_html__( 'NONE', 'woocommerce-dhl24' );
		}

		return esc_html( $courier_order_id );
	}

	/**
	 * Escape company address.
	 *
	 * @return string
	 */
	public function esc_company_address(): string {
		$data = Option::get_instance()->get_data();

		$xhtml = esc_html( $data['street'] . ' ' . $data['house_nmber'] )
			. ( $data['apartment_number'] ? '/' . esc_html( $data['apartment_number'] ) : '' )
			. '<br />'
			. esc_html( $data['postal_code'] . ' ' . $data['city'] )
			. '<br />'
			. esc_html( $data['contact_person'] )
			. '<br />'
			. esc_html( $data['contact_phone'] . ', ' . $data['contact_email'] );

		return $xhtml;
	}

	/**
	 * Escape company address.
	 *
	 * @return string
	 */
	public function esc_company_return_address(): string {
		$data = Option::get_instance()->get_data();

		$xhtml = esc_html( $data['return_street'] . ' ' . $data['return_house_nmber'] )
			. ( $data['return_apartment_number'] ? '/' . esc_html( $data['return_apartment_number'] ) : '' )
			. '<br />'
			. esc_html( $data['return_postal_code'] . ' ' . $data['return_city'] )
			. '<br />'
			. esc_html( $this->get_coutry_name( $data['return_country'] ) )
			. '<br />'
			. esc_html( $data['return_contact_person'] )
			. '<br />'
			. esc_html( $data['return_contact_phone'] . ', ' . $data['return_contact_email'] );

		return $xhtml;
	}

	/**
	 * Escape shipper address.
	 *
	 * @param ShipperAddressLabel|null $address ShipperAddressLabel.
	 * @param string|null              $service_type Service type.
	 * @return string
	 */
	public function esc_sender_address( $address, $service_type = null ): string {
		if ( ! $address instanceof ShipperAddressLabel ) {
			return '';
		}

		$xhtml = esc_html( $address->getStreet() . ' ' . $address->getHouseNumber() )
			. ( $address->getApartmentNumber() ? '/' . esc_html( $address->getApartmentNumber() ) : '' )
			. '<br />'
			. esc_html( $address->getPostalCode() . ' ' . $address->getCity() );
		if ( in_array( $service_type, array( 'ZK', 'ZC' ), true ) ) {
			$xhtml .= '<br />'
			. esc_html( $address->getContactPhone() . ', ' . $address->getContactEmail() );
		}

		return $xhtml;
	}

	/**
	 * Escape WP shipping address.
	 *
	 * @param array $shipping Shippin address.
	 * @return string
	 */
	public function esc_wp_shipping_address( array $shipping ): string {
		$xhtml = esc_html( $shipping['address_1'] )
			. ( $shipping['address_2'] ? '/' . esc_html( $shipping['address_2'] ) : '' )
			. '<br />'
			. esc_html( $shipping['postcode'] . ' ' . $shipping['city'] )
			. '<br />'
			. esc_html( $this->get_coutry_name( $shipping['country'] ) );

		return $xhtml;
	}

	/**
	 * Escape receiver address.
	 *
	 * @param ReceiverAddressLabel|null $address ReceiverAddressLabel.
	 * @return string
	 */
	public function esc_recipient_address( $address ): string {
		if ( ! $address instanceof ReceiverAddressLabel ) {
			return '';
		}

		$xhtml = esc_html( $address->getStreet() . ' ' . $address->getHouseNumber() )
			. ( $address->getApartmentNumber() ? '/' . esc_html( $address->getApartmentNumber() ) : '' )
			. '<br />'
			. esc_html( $address->getPostalCode() . ' ' . $address->getCity() )
			. '<br />'
			. esc_html( $this->get_coutry_name( $address->getCountry() ) )
			. '<br />'
			. '<br />'
			. esc_html( $address->getContactPhone() . ', ' . $address->getContactEmail() );

		return $xhtml;
	}

	/**
	 * Escape billing information.
	 *
	 * @param Billing|null $billing Billing.
	 * @return string
	 */
	public function esc_billing( $billing ): string {
		if ( ! $billing instanceof Billing ) {
			return '';
		}
		$xhtml = '<br />'
			. esc_html__( 'Payer', 'woocommerce-dhl24' )
			. ': '
			. esc_html( $billing->getBillingAccountNumber() )
			. '<br />'
			. esc_html__( 'Payment form', 'woocommerce-dhl24' )
			. ': ';
		if ( 'BANK_TRANSFER' === $billing->getPaymentType() ) {
			$xhtml .= esc_html__( 'Bank transfer', 'woocommerce-dhl24' );
		} elseif ( 'CASH' === $billing->getPaymentType() ) {
			$xhtml .= esc_html__( 'Cash payment', 'woocommerce-dhl24' );
		} else {
			$xhtml .= esc_html( $billing->getPaymentType() );
		}

		return $xhtml;
	}

	/**
	 * Escape parcel quantity.
	 *
	 * @param string $type Parcel type.
	 * @param int    $quantity Quantity.
	 * @return string
	 */
	public function esc_parcel_quantity( string $type, int $quantity ): string {
		if ( 'ENVELOPE' === $type ) {
			return esc_html__( 'Envelopes', 'woocommerce-dhl24' ) . ': ' . $quantity;
		} elseif ( 'PACKAGE' === $type ) {
			return esc_html__( 'Parcels', 'woocommerce-dhl24' ) . ': ' . $quantity;
		} elseif ( 'PALLET' === $type ) {
			return esc_html__( 'Pallets', 'woocommerce-dhl24' ) . ': ' . $quantity;
		} elseif ( 'weight' === $type ) {
			return esc_html__( 'Sum weight of the parcels', 'woocommerce-dhl24' ) . ': ' . $quantity . ' kg';
		}

		return esc_html( $type ) . ': ' . $quantity;
	}

	/**
	 * Escape special services.
	 *
	 * @param  LabelDataServiceDefinition|null $service LabelDataServiceDefinition.
	 * @return string
	 */
	public function esc_special_services( $service ): string {
		if ( ! $service instanceof LabelDataServiceDefinition ) {
			return '';
		}

		$services = array();
		if ( true === $service->getDeliveryEvening() ) {
			$services[] = esc_html__( 'Evening delivery', 'woocommerce-dhl24' );
		}
		if ( true === $service->getDeliveryOnSaturday() ) {
			$services[] = esc_html__( 'Evening on Saturday', 'woocommerce-dhl24' );
		}
		if ( true === $service->getPickupOnSaturday() ) {
			$services[] = esc_html__( 'Pickup on Saturday', 'woocommerce-dhl24' );
		}
		if ( true === $service->getCollectOnDelivery() ) {
			/* translators: %s: Collection amount in PLN */
			$services[] = sprintf( esc_html__( 'Collect on delivery (COD): %s', 'woocommerce-dhl24' ), esc_html( $this->format_price( $service->getCollectOnDeliveryValue() ) ) );
			$services[] = esc_html( $service->getCollectOnDeliveryForm() );
			$services[] = esc_html( $service->getCollectOnDeliveryReference() );
		}
		if ( true === $service->getInsurance() ) {
			/* translators: %s: Insurance amount in PLN */
			$services[] = sprintf( esc_html__( 'Shipment insurance: %s', 'woocommerce-dhl24' ), esc_html( $this->format_price( $service->getInsuranceValue() ) ) );
		}
		if ( true === $service->getReturnOnDelivery() ) {
			$services[] = esc_html__( 'Return of documents', 'woocommerce-dhl24' );
			$services[] = esc_html( $service->getReturnOnDeliveryReference() );
		}
		if ( true === $service->getProofOfDelivery() ) {
			$services[] = esc_html__( 'Proof of delivery', 'woocommerce-dhl24' );
		}
		if ( true === $service->getSelfCollect() ) {
			$services[] = esc_html__( 'Self pickup', 'woocommerce-dhl24' );
		}
		if ( true === $service->getPredeliveryInformation() ) {
			$services[] = esc_html__( 'Pre delivery information', 'woocommerce-dhl24' );
		}
		if ( true === $service->getDeliveryToNeighbour() ) {
			$services[] = esc_html__( 'Delivery to neighbour', 'woocommerce-dhl24' );
		}
		return implode( '<br />', $services );
	}

	/**
	 * Esc add piece row
	 *
	 * @param  string $shipping_method Shipping method.
	 * @param  string $index Row index.
	 * @param  array  $values Values.
	 * @return string
	 */
	public function esc_add_piece_row( string $shipping_method, string $index = '0', array $values = array() ): string {
		$t_quantity      = esc_attr__( 'Quantity', 'woocommerce-dhl24' );
		$t_weight        = esc_attr__( 'Weight', 'woocommerce-dhl24' );
		$t_length        = esc_attr__( 'Length', 'woocommerce-dhl24' );
		$t_width         = esc_attr__( 'Width', 'woocommerce-dhl24' );
		$t_height        = esc_attr__( 'Height', 'woocommerce-dhl24' );
		$t_non_standard  = esc_attr__( 'Non-standard', 'woocommerce-dhl24' );
		$t_pallet_return = esc_attr__( 'Pallet return', 'woocommerce-dhl24' );
		$t_delete        = esc_attr__( 'Delete', 'woocommerce-dhl24' );
		$t_add           = esc_attr__( 'Add', 'woocommerce-dhl24' );
		$t_envelope      = esc_attr__( 'Envelope', 'woocommerce-dhl24' );
		$t_package       = esc_attr__( 'Package', 'woocommerce-dhl24' );
		$t_palett        = esc_attr__( 'Palett', 'woocommerce-dhl24' );

		$defaults_data = Option::get_instance()->get_data();
		if ( empty( $values ) ) {
			$v_quantity     = esc_attr( $defaults_data['piece_quantity'] );
			$v_weight       = esc_attr( $defaults_data['piece_weight'] );
			$v_length       = esc_attr( $defaults_data['piece_length'] );
			$v_width        = esc_attr( $defaults_data['piece_width'] );
			$v_height       = esc_attr( $defaults_data['piece_height'] );
			$v_type         = esc_attr( $defaults_data['piece_type'] );
			$v_non_standard = '';
			$v_euro_return  = '';
		} else {
			$v_quantity     = esc_attr( $values['piece_quantity'] );
			$v_weight       = esc_attr( $values['piece_weight'] );
			$v_length       = esc_attr( $values['piece_length'] );
			$v_width        = esc_attr( $values['piece_width'] );
			$v_height       = esc_attr( $values['piece_height'] );
			$v_type         = esc_attr( $values['piece_type'] );
			$v_non_standard = true === $values['piece_non_standard'] ? ' checked="checked"' : '';
			$v_euro_return  = true === $values['piece_euro_return'] ? ' checked="checked"' : '';
		}
		$v_selected_1 = 'ENVELOPE' === $v_type ? ' selected="selected"' : '';
		$v_selected_2 = 'PACKAGE' === $v_type ? ' selected="selected"' : '';
		$v_selected_3 = 'PALLET' === $v_type ? ' selected="selected"' : '';

		$servicepoint_shipping_methods = Param::get_instance()->get_value( 'servicepoint_shipping_methods' );
		$return_shipping_methods       = Param::get_instance()->get_value( 'return_shipping_methods' );
		$allow_palet_shipping_methods  = array_merge( $servicepoint_shipping_methods, $return_shipping_methods );

		$value_name = 'new_shipment';
		if ( in_array( $shipping_method, $return_shipping_methods, true ) ) {
			$value_name = 'return_shipment';
		}

		$xhtml = <<<EOT
<tr>
	<td><input class="dhl24_piece_quantity" type="text" name="{$value_name}[piece][{$index}][quantity]" value="{$v_quantity}" placeholder="{$t_quantity} *" required="required"></td>
	<td>x</td>
	<td><select class="dhl24_piece_type" name="{$value_name}[piece][{$index}][type]">
		<option value="ENVELOPE"{$v_selected_1}>{$t_envelope}</option>
		<option value="PACKAGE"{$v_selected_2}>{$t_package}</option>
EOT;

		if ( ! in_array( $shipping_method, $allow_palet_shipping_methods, true ) ) {
			$xhtml .= '<option value="PALLET"' . $v_selected_3 . '>' . $t_palett . '</option>';
		}
		$xhtml .= <<<EOT
	</select></td>
	<td>x</td>
	<td class="dhl24-size-{$index}"><input class="dhl24_piece_weight" type="text" name="{$value_name}[piece][{$index}][weight]" value="{$v_weight}" placeholder="{$t_weight}"></td>
	<td>kg</td>
	<td class="dhl24-size-{$index}"><input class="dhl24_piece_length" type="text" name="{$value_name}[piece][{$index}][length]" value="{$v_length}" placeholder="{$t_length}"></td>
	<td>x</td>
	<td class="dhl24-size-{$index}"><input class="dhl24_piece_width" type="text" name="{$value_name}[piece][{$index}][width]" value="{$v_width}" placeholder="{$t_width}"></td>
	<td>x</td>
	<td class="dhl24-size-{$index}"><input class="dhl24_piece_height" type="text" name="{$value_name}[piece][{$index}][height]" value="{$v_height}" placeholder="{$t_height}"></td>
	<td>cm</td>
EOT;
		if ( ! in_array( $shipping_method, $servicepoint_shipping_methods, true ) ) {
			$xhtml .= <<<EOT
	<td class="dhl24-size-{$index} checkbox"><input class="dhl24_piece_nonStandard" type="checkbox" name="{$value_name}[piece][{$index}][nonStandard]" value="1" id="piecie_{$index}_nonStandard" {$v_non_standard} /><label for="piecie_{$index}_nonStandard">{$t_non_standard}</label></td>
EOT;
		}
		if ( ! in_array( $shipping_method, $allow_palet_shipping_methods, true ) ) {
			$xhtml .= <<<EOT
	<td class="dhl24-pallet-{$index} checkbox"><input class="dhl24_piece_euroReturn" type="checkbox" name="{$value_name}[piece][{$index}][euroReturn]" value="1" id="piecie_{$index}_euroReturn" {$v_euro_return} /><label for="piecie_{$index}_euroReturn">{$t_pallet_return}</label></td>
EOT;
		}
		$display_delete = ' style="display:none"';
		if ( ! empty( $index ) ) {
			$display_delete = '';
		}
		$xhtml .= <<<EOT
	<td class="action">
		<button type="button" class="dhl24-delete-piece"{$display_delete}>{$t_delete}</button>
		<button type="button" class="dhl24-add-piece">{$t_add}</button>
	</td>
</tr>
EOT;
		return $xhtml;
	}

	/**
	 * Format price
	 *
	 * @param  float|null  $price Price.
	 * @param  string|null $currency Currency.
	 * @return string
	 */
	public function format_price( $price, $currency = Defaults::CURRENCY ): string {
		if ( null === $price ) {
			$price = 0;
		}
		if ( null === $currency ) {
			$currency = Defaults::CURRENCY;
		}
		return number_format( $price, 2, ',', ' ' ) . ' ' . $currency;
	}

	/**
	 * Url
	 *
	 * @return string
	 */
	public function admin_url(): string {
		$url = admin_url( 'admin.php?page=' . Defaults::ADMIN_PAGE );
		return wp_make_link_relative( $url );
	}

	/**
	 * Url
	 *
	 * @return string
	 */
	public function admin_post_url(): string {
		$url = admin_url( 'admin-post.php' );
		return wp_make_link_relative( $url );
	}

	/**
	 * Download label url
	 *
	 * @param  string $shipment_id Shipment ID.
	 * @param  string $label_type Label type.
	 * @param  bool   $is_servicepoint_label Is servicepoint label.
	 * @return string
	 */
	public function download_label_url( string $shipment_id, string $label_type, bool $is_servicepoint_label = false ): string {
		$servicepoint_label = '';
		if ( true === $is_servicepoint_label ) {
			$servicepoint_label = '&' . Defaults::SHIPMENT_TYPE . '=lm';
		}
		return '/?' . Defaults::DOWNLOAD_LABEL . '=' . $shipment_id . '&' . Defaults::LABEL_TYPE . '=' . $label_type . $servicepoint_label;
	}

	/**
	 * Download scan url
	 *
	 * @param  string $shipment_id Shipment ID.
	 * @return string
	 */
	public function download_scan_url( string $shipment_id ): string {
		return '/?' . Defaults::DOWNLOAD_SCAN . '=' . $shipment_id;
	}

	/**
	 * Get page.
	 *
	 * @return string
	 */
	public function get_page(): string {
		return Defaults::ADMIN_PAGE;
	}

	/**
	 * Meta box links
	 *
	 * @param  string $shipment_id Shipment ID.
	 * @param  string $dispatch_number Dispatch number.
	 * @param  bool   $is_servicepoint_label Is servicepoint label.
	 * @return string
	 */
	public function meta_box_links( string $shipment_id, string $dispatch_number, bool $is_servicepoint_label = false ): string {
		$xhtml = '';
		if ( ! empty( $shipment_id ) ) {
			$xhtml .= '<p><a href="'
			. esc_url(
				add_query_arg(
					array(
						'path' => '/shipment',
						'id'   => $shipment_id,
					),
					$this->admin_url()
				)
			)
			. '">'
			. esc_html__( 'Show details of shipment', 'woocommerce-dhl24' )
			. '</a></p>'
			. '<p><a href="'
			. esc_url( $this->download_label_url( $shipment_id, 'lblp', $is_servicepoint_label ) )
			. '">'
			. esc_html__( 'Download BLP label in A4 PDF format', 'woocommerce-dhl24' )
			. '</a></p>'
			. '<p><a href="'
			. esc_url( $this->download_label_url( $shipment_id, 'zblp', $is_servicepoint_label ) )
			. '">'
			. esc_html__( 'Download BLP label in ZPL format', 'woocommerce-dhl24' )
			. '</a></p>'
			. '<p><a href="'
			. esc_url( $this->download_label_url( $shipment_id, 'blp', $is_servicepoint_label ) )
			. '">'
			. esc_html__( 'Download BLP label', 'woocommerce-dhl24' )
			. '</a></p>'
			. '<p><a href="'
			. esc_url( $this->download_scan_url( $shipment_id ) )
			. '">'
			. esc_html__( 'Download a scan of the consignment note for a completed shipment', 'woocommerce-dhl24' )
			. '</a></p>';
			$xhtml .= '<p><a href="'
			. esc_url(
				add_query_arg(
					array(
						'path'  => '/delete_shipment',
						'id'    => $shipment_id,
						'co_id' => $dispatch_number,
					),
					$this->admin_url()
				)
			)
			. '">'
			. esc_html__( 'Delete shipment', 'woocommerce-dhl24' )
			. '</a></p>';
		}

		return $xhtml;
	}

	/**
	 * Displays parcels pagination
	 *
	 * @param  string $which Which: top or button.
	 * @param  int    $total_items Total items.
	 * @param  int    $current Current.
	 * @param  bool   $infinite_scroll Infinite scroll.
	 * @return void
	 */
	public function parcels_pagination( string $which, int $total_items, int $current, bool $infinite_scroll = false ) {
		$total_pages = (int) ceil( $total_items / 100 );

		$output = '<span class="displaying-num">' . sprintf(
			/* translators: %s: Number of items. */
			_n( '%s item', '%s items', $total_items ), // phpcs:ignore WordPress.WP.I18n.MissingArgDomain
			number_format_i18n( $total_items )
		) . '</span>';

		$current_url        = $this->admin_url();
		$total_pages_before = '<span class="paging-input">';
		$total_pages_after  = '</span></span>';

		$disable_first = false;
		$disable_last  = false;
		$disable_prev  = false;
		$disable_next  = false;

		if ( 1 === $current ) {
			$disable_first = true;
			$disable_prev  = true;
		}
		if ( 2 === $current ) {
			$disable_first = true;
		}
		if ( $total_pages === $current ) {
			$disable_last = true;
			$disable_next = true;
		}
		if ( $total_pages - 1 === $current ) {
			$disable_last = true;
		}

		if ( $disable_first ) {
			$page_links[] = '<span class="tablenav-pages-navspan button disabled" aria-hidden="true">&laquo;</span>';
		} else {
			$page_links[] = sprintf(
				"<a class='first-page button' href='%s'><span class='screen-reader-text'>%s</span><span aria-hidden='true'>%s</span></a>",
				esc_url( remove_query_arg( 'paged', $current_url ) ),
				__( 'First page' ), // phpcs:ignore WordPress.WP.I18n.MissingArgDomain
				'&laquo;'
			);
		}

		if ( $disable_prev ) {
			$page_links[] = '<span class="tablenav-pages-navspan button disabled" aria-hidden="true">&lsaquo;</span>';
		} else {
			$page_links[] = sprintf(
				"<a class='prev-page button' href='%s'><span class='screen-reader-text'>%s</span><span aria-hidden='true'>%s</span></a>",
				esc_url( add_query_arg( 'paged', max( 1, $current - 1 ), $current_url ) ),
				__( 'Previous page' ), // phpcs:ignore WordPress.WP.I18n.MissingArgDomain
				'&lsaquo;'
			);
		}

		if ( 'bottom' === $which ) {
			$html_current_page  = $current;
			$total_pages_before = '<span class="screen-reader-text">' . __( 'Current Page' ) . '</span><span id="table-paging" class="paging-input"><span class="tablenav-paging-text">'; // phpcs:ignore WordPress.WP.I18n.MissingArgDomain
		} else {
			$html_current_page = sprintf(
				"%s<input class='current-page' id='current-page-selector' type='text' name='paged' value='%s' size='%d' aria-describedby='table-paging' /><span class='tablenav-paging-text'>",
				'<label for="current-page-selector" class="screen-reader-text">' . __( 'Current Page' ) . '</label>', // phpcs:ignore WordPress.WP.I18n.MissingArgDomain
				$current,
				strlen( $total_pages )
			);
		}
		$html_total_pages = sprintf( "<span class='total-pages'>%s</span>", number_format_i18n( $total_pages ) );
		$page_links[]     = $total_pages_before . sprintf(
			/* translators: 1: Current page, 2: Total pages. */
			_x( '%1$s of %2$s', 'paging' ), // phpcs:ignore WordPress.WP.I18n.MissingArgDomain
			$html_current_page,
			$html_total_pages
		) . $total_pages_after;

		if ( $disable_next ) {
			$page_links[] = '<span class="tablenav-pages-navspan button disabled" aria-hidden="true">&rsaquo;</span>';
		} else {
			$page_links[] = sprintf(
				"<a class='next-page button' href='%s'><span class='screen-reader-text'>%s</span><span aria-hidden='true'>%s</span></a>",
				esc_url( add_query_arg( 'paged', min( $total_pages, $current + 1 ), $current_url ) ),
				__( 'Next page' ), // phpcs:ignore WordPress.WP.I18n.MissingArgDomain
				'&rsaquo;'
			);
		}

		if ( $disable_last ) {
			$page_links[] = '<span class="tablenav-pages-navspan button disabled" aria-hidden="true">&raquo;</span>';
		} else {
			$page_links[] = sprintf(
				"<a class='last-page button' href='%s'><span class='screen-reader-text'>%s</span><span aria-hidden='true'>%s</span></a>",
				esc_url( add_query_arg( 'paged', $total_pages, $current_url ) ),
				__( 'Last page' ), // phpcs:ignore WordPress.WP.I18n.MissingArgDomain
				'&raquo;'
			);
		}

		$pagination_links_class = 'pagination-links';
		if ( ! empty( $infinite_scroll ) ) {
			$pagination_links_class .= ' hide-if-js';
		}
		$output .= "\n<span class='$pagination_links_class'>" . implode( "\n", $page_links ) . '</span>';

		if ( $total_pages ) {
			$page_class = $total_pages < 2 ? ' one-page' : '';
		} else {
			$page_class = ' no-pages';
		}
		$pagination = "<div class='tablenav-pages{$page_class}'>$output</div>";

		echo $pagination; // phpcs:ignore
	}
}
