<?php
/**
 * Class ShipmentRepository.
 *
 * @package ILabs\WoocommerceDHL24
 */

namespace ILabs\WoocommerceDHL24\Repository;

use ILabs\WoocommerceDHL24\Helper\Option;
use ILabs\WoocommerceDHL24\Exception\RepositoryException;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\Shipment;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\Shipments;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\LabelsItem;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Method\GetLabels;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ItemToPrint;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\LabelsDatas;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ScanToPrint;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\TrackAndTrace;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Method\GetLabelsData;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ReturnShipments;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Method\CreateShipment;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Method\DeleteShipment;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ItemToLabelData;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Method\GetShipmentScan;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ReturnItemToPrint;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\DeleteNotifications;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Method\GetReturnByWaybill;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Method\CancelCourierBooking;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Method\CreateShipmentReturn;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Method\GetTrackAndTraceInfo;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\DeleteShipment as DeleteShipmentResponse;

/**
 * ShipmentRepository
 */
class ShipmentRepository extends AbstractRepository {
	/**
	 * Fetch label print out
	 *
	 * @param  string $shipment_id Shipment number of the shipment for which information is to be downloaded.
	 * @param  string $label_type Label type.
	 * @return LabelsItem
	 * @throws RepositoryException If error.
	 */
	public function fetch_label_print_out( string $shipment_id, string $label_type ): LabelsItem {
		$params        = new GetLabels( $this->auth );
		$item_to_print = new ItemToPrint();
		$item_to_print->setLabelType( $label_type );
		$item_to_print->setShipmentId( $shipment_id );
		$params->addItemToPrint( $item_to_print );

		try {
			$labels = $this->web_api_client->getLabels( $params );
		} catch ( \Exception $e ) {
			throw new RepositoryException( $e, $e->getMessage() );
		}

		if ( 1 !== $labels->getItems()->count() ) {
			throw new RepositoryException( null, __( 'More than 1 label.', 'woocommerce-dhl24' ) );
		}
		return $labels->getItems()->offsetGet( 0 );
	}

	/**
	 * Fetch information on labels
	 *
	 * @param  string $shipment_id Shipment number of the shipment for which information is to be downloaded.
	 * @return LabelsDatas
	 * @throws RepositoryException If error.
	 */
	public function fetch_label( string $shipment_id ): LabelsDatas {
		$params              = new GetLabelsData( $this->auth );
		$piece_shipment_item = new ItemToLabelData();
		$piece_shipment_item->setShipmentId( $shipment_id );
		$params->addItemToLabelData( $piece_shipment_item );

		try {
			$labels = $this->web_api_client->getLabelsData( $params );
		} catch ( \Exception $e ) {
			throw new RepositoryException( $e, $e->getMessage() );
		}

		return $labels;
	}

	/**
	 * Fetch shipment scan
	 *
	 * @param  string $shipment_id Shipment number of the shipment for which information is to be downloaded.
	 * @return ScanToPrint
	 * @throws RepositoryException If error.
	 */
	public function fetch_scan( string $shipment_id ): ScanToPrint {
		$params = new GetShipmentScan( $this->auth );
		$params->setShipmentId( $shipment_id );

		try {
			$labels = $this->web_api_client->getShipmentScan( $params );
		} catch ( \Exception $e ) {
			throw new RepositoryException( $e, $e->getMessage() );
		}

		return $labels;
	}

	/**
	 * Fetch information on track and trace
	 *
	 * @param  string $shipment_id Shipment number of the shipment for which information is to be downloaded.
	 * @return TrackAndTrace
	 * @throws RepositoryException If error.
	 */
	public function fetch_tracking( string $shipment_id ): TrackAndTrace {
		$params = new GetTrackAndTraceInfo( $this->auth );
		$params->setShipmentId( $shipment_id );

		try {
			$track_and_trace = $this->web_api_client->getTrackAndTraceInfo( $params );
		} catch ( \Exception $e ) {
			throw new RepositoryException( $e, $e->getMessage() );
		}

		return $track_and_trace;
	}

	/**
	 * Cancel courier booking
	 *
	 * @param  string $courier_order_id Courier order number.
	 * @return DeleteNotifications
	 * @throws RepositoryException If error.
	 */
	public function cancel_courier_booking( string $courier_order_id ): DeleteNotifications {
		$params = new CancelCourierBooking( $this->auth );
		$params->addOrder( $courier_order_id );

		try {
			$delete_notifications = $this->web_api_client->cancelCourierBooking( $params );
		} catch ( \Exception $e ) {
			throw new RepositoryException( $e, $e->getMessage() );
		}

		return $delete_notifications;
	}

	/**
	 * Delete shipment
	 *
	 * @param  string $shipment_id Shipment number of the shipment to be deleted.
	 * @param  string $courier_order_id Courier order ID.
	 * @return DeleteShipmentResponse
	 * @throws RepositoryException If error.
	 */
	public function delete( string $shipment_id, string $courier_order_id ): DeleteShipmentResponse {
		$params = new DeleteShipment( $this->auth );
		$params->setShipmentIdentificationNumber( $shipment_id );
		$params->setDispatchIdentificationNumber( $courier_order_id );

		try {
			$delete_notifications = $this->web_api_client->deleteShipment( $params );
		} catch ( \Exception $e ) {
			throw new RepositoryException( $e, $e->getMessage() );
		}

		return $delete_notifications;
	}

	/**
	 * Create shipment
	 *
	 * @param  array $shipment_data Array of shipment data.
	 * @return Shipment
	 * @throws RepositoryException If error.
	 */
	public function create_shipment( array $shipment_data ):Shipment {
		$params = new CreateShipment( $this->auth );
		$params->setShip( $shipment_data['ship'] );
		$params->setShipmentInfo( $shipment_data['shipment_info'] );
		foreach ( $shipment_data['pieces'] as $piece ) {
			$params->addPiece( $piece );
		}
		$params->setContent( $shipment_data['content'] );
		$params->setComment( $shipment_data['comment'] );
		$params->setReference( $shipment_data['reference'] );

		try {
			$shipment = $this->web_api_client->createShipment( $params );
		} catch ( \Exception $e ) {
			throw new RepositoryException( $e, $e->getMessage() );
		}

		return $shipment;
	}

	/**
	 * Return shipment
	 *
	 * @param  string $shipment_id Shipment number of the shipment to be returned.
	 * @return Shipments
	 * @throws RepositoryException If error.
	 */
	public function return_shipment( string $shipment_id ):Shipments {
		$params        = new GetReturnByWaybill( $this->auth );
		$item_to_print = new ReturnItemToPrint();
		$item_to_print->setShipmentId( $shipment_id );
		$item_to_print->setLabelType( Option::get_instance()->get_value( 'label_type' ) );
		$params->addItemToPrint( $item_to_print );

		try {
			$shipments = $this->web_api_client->getReturnByWaybill( $params );
		} catch ( \Exception $e ) {
			throw new RepositoryException( $e, $e->getMessage() );
		}

		return $shipments;
	}

	/**
	 * Create shipment return
	 *
	 * @param  array $shipment_data Array of shipment data.
	 * @return ReturnShipments
	 * @throws RepositoryException If error.
	 */
	public function create_shipment_return( array $shipment_data ):ReturnShipments {
		$params = new CreateShipmentReturn( $this->auth );
		$params->setShip( $shipment_data['ship'] );
		$params->setShipmentInfo( $shipment_data['shipment_info'] );
		foreach ( $shipment_data['pieces'] as $piece ) {
			$params->addPiece( $piece );
		}
		$params->setContent( $shipment_data['content'] );
		$params->setComment( $shipment_data['comment'] );
		$params->setReference( $shipment_data['reference'] );
		$params->setPrimaryWaybillNumber( $shipment_data['primary_waybill_number'] );

		try {
			$return_shipments = $this->web_api_client->createShipmentReturn( $params );
		} catch ( \Exception $e ) {
			throw new RepositoryException( $e, $e->getMessage() );
		}

		return $return_shipments;
	}
}
