<?php
/**
 * Class ShipmentsRepository.
 *
 * @package ILabs\WoocommerceDHL24
 */

namespace ILabs\WoocommerceDHL24\Repository;

use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\Pnp;
use ILabs\WoocommerceDHL24\Exception\RepositoryException;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\Strings;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Method\GetPnp;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\LabelsDatas;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Method\GetPieceId;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Method\BookCourier;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\PieceShipments;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Method\GetLabelsData;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ItemToLabelData;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Method\GetMyShipments;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ShipmentFullData;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Method\CreateShipments;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\PieceShipmentItem;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ShipmentBasicData;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ShipmentBasicDatas;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\NearestServicepoints;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Method\GetMyShipmentsCount;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Method\GetNearestServicepoints;

/**
 * ShipmentsRepository
 */
class ShipmentsRepository extends AbstractRepository {

	/**
	 * Start of time range, date in YYYY-MM-DD format.
	 *
	 * @var \DateTime
	 */
	private $from;
	/**
	 * End of time range, date in YYYY-MM-DD format.
	 *
	 * @var \DateTime
	 */
	private $to;

	/**
	 * Get start of time range, date in YYYY-MM-DD format.
	 *
	 * @return  \DateTime
	 */
	public function get_from() {
		return $this->from;
	}

	/**
	 * Set start of time range, date in YYYY-MM-DD format.
	 *
	 * @param  \DateTime $from  Start of time range, date in YYYY-MM-DD format.
	 *
	 * @return  self
	 */
	public function set_from( \DateTime $from ) {
		$this->from = $from;

		return $this;
	}

	/**
	 * Get end of time range, date in YYYY-MM-DD format.
	 *
	 * @return  \DateTime
	 */
	public function get_to() {
		return $this->to;
	}

	/**
	 * Set end of time range, date in YYYY-MM-DD format.
	 *
	 * @param  \DateTime $to  End of time range, date in YYYY-MM-DD format.
	 *
	 * @return  self
	 */
	public function set_to( \DateTime $to ) {
		$this->to = $to;

		return $this;
	}

	/**
	 * Fetch my shipments count
	 *
	 * @return int
	 * @throws RepositoryException If error.
	 */
	public function fetch_my_shipments_count(): int {
		$params = new GetMyShipmentsCount( $this->auth );
		if ( null !== $this->from ) {
			$params->setCreatedFrom( $this->from );
		}
		if ( null !== $this->to ) {
			$params->setCreatedTo( $this->to );
		}

		try {
			$my_shipments_count = $this->web_api_client->getMyShipmentsCount( $params );
			$my_shipments_count = $my_shipments_count->getShipmentsCount();
		} catch ( \Exception $e ) {
			throw new RepositoryException( $e, $e->getMessage() );
		}

		return $my_shipments_count;
	}

	/**
	 * Fetch my shipments
	 *
	 * @param int $offset The service returns the maximum of 100 records - use an offset if the number of records in the time slot is exceeded.
	 * @return ShipmentBasicDatas
	 * @throws RepositoryException If error.
	 */
	public function fetch_my_shipments( int $offset = 0 ): ShipmentBasicDatas {
		$params = new GetMyShipments( $this->auth );
		if ( null !== $this->from ) {
			$params->setCreatedFrom( $this->from );
		}
		if ( null !== $this->to ) {
			$params->setCreatedTo( $this->to );
		}
		if ( 0 < $offset ) {
			$offset = $offset * 100;
			$params->setOffset( $offset );
		}

		try {
			$my_shipments = $this->web_api_client->getMyShipments( $params );
		} catch ( \Exception $e ) {
			throw new RepositoryException( $e, $e->getMessage() );
		}

		return $my_shipments;
	}

	/**
	 * Fetch information on pieces
	 *
	 * @param  \ArrayObject $shipments Array of ShipmentBasicData.
	 * @return PieceShipments
	 * @throws RepositoryException If error.
	 */
	public function fetch_pieces( \ArrayObject $shipments ): PieceShipments {
		$params = new GetPieceId( $this->auth );
		/**
		 * Loop
		 *
		 * @var ShipmentBasicData $shipment ShipmentBasicData.
		 */
		foreach ( $shipments as $shipment ) {
			$piece_shipment_item = new PieceShipmentItem();
			$piece_shipment_item->setShipmentNumber( $shipment->getShipmentId() );
			$params->addItem( $piece_shipment_item );
		}

		try {
			$pieces = $this->web_api_client->getPieceId( $params );
		} catch ( \Exception $e ) {
			throw new RepositoryException( $e, $e->getMessage() );
		}

		return $pieces;
	}

	/**
	 * Fetch information on labels
	 *
	 * @param  array $shipment_id_list Array of shipment_id.
	 * @return LabelsDatas
	 * @throws RepositoryException If error.
	 */
	public function fetch_labels( array $shipment_id_list ): LabelsDatas {
		if ( 0 === count( $shipment_id_list ) ) {
			return new LabelsDatas();
		}

		$params = new GetLabelsData( $this->auth );
		/**
		 * Loop
		 *
		 * @var ShipmentBasicData $shipment ShipmentBasicData.
		 */
		foreach ( $shipment_id_list as $shipment_id ) {
			if ( ! empty( $shipment_id ) ) {
				$piece_shipment_item = new ItemToLabelData();
				$piece_shipment_item->setShipmentId( $shipment_id );
				$params->addItemToLabelData( $piece_shipment_item );
			}
		}

		try {
			$labels = $this->web_api_client->getLabelsData( $params );
		} catch ( \Exception $e ) {
			throw new RepositoryException( $e, $e->getMessage() );
		}

		return $labels;
	}

	/**
	 * Book courier
	 *
	 * @param  array     $ids Array of shipment IDs.
	 * @param  \DateTime $pickup_date Pickup date.
	 * @param  string    $from Time from which the shipment is ready to be collected (in HH:MM format).
	 * @param  string    $to Time until which the shipment can be collected (in HH:MM format).
	 * @param  string    $comment Additional information for the courier.
	 * @param  bool      $with_label Should the courier come with a label.
	 * @return Strings
	 * @throws RepositoryException If error.
	 */
	public function book_courier( array $ids, \DateTime $pickup_date, string $from, string $to, string $comment = '', bool $with_label = false ): Strings {
		$params = new BookCourier( $this->auth );
		$params->setPickupDate( $pickup_date );
		$params->setPickupTimeFrom( $from );
		$params->setPickupTimeTo( $to );
		$params->setAdditionalInfo( $comment );
		$params->setCourierWithLabel( $with_label );
		foreach ( $ids as $shipment_id ) {
			$params->addShipmentId( $shipment_id );
		}

		try {
			$book_ids = $this->web_api_client->bookCourier( $params );
		} catch ( \Exception $e ) {
			throw new RepositoryException( $e, $e->getMessage() );
		}

		return $book_ids;
	}

	/**
	 * Fetch nearest service points
	 *
	 * @param  string      $post_code Postal code.
	 * @param  string|null $country Two-letter country code.
	 * @param  int         $radius Radius.
	 * @return NearestServicepoints
	 * @throws RepositoryException If error.
	 */
	public function fetch_nearest_service_points( $post_code, $country, int $radius = 10 ): NearestServicepoints {
		if ( empty( $country ) ) {
			$country = 'PL';
		}
		$post_code = $this->formatter->post_code( $post_code, $country );

		$params = new GetNearestServicepoints( $this->auth );
		$params->setCountry( $country );
		$params->setPostcode( $post_code );
		$params->setRadius( $radius );

		try {
			$nearest_service_points = $this->web_api_client->getNearestServicepoints( $params );
		} catch ( \Exception $e ) {
			throw new RepositoryException( $e, $e->getMessage() );
		}

		return $nearest_service_points;
	}

	/**
	 * Create shipments
	 *
	 * @param  array|ShipmentFullData[] $shipments Array of ShipmentFullData.
	 * @return ShipmentBasicDatas
	 * @throws RepositoryException If error.
	 */
	public function create_shipments( array $shipments ):ShipmentBasicDatas {
		$params = new CreateShipments( $this->auth );
		foreach ( $shipments as $shipment ) {
			$params->addShipment( $shipment );
		}

		try {
			$shipment_basic_datas = $this->web_api_client->createShipments( $params );
		} catch ( \Exception $e ) {
			throw new RepositoryException( $e, $e->getMessage() );
		}

		return $shipment_basic_datas;
	}

	/**
	 * Get pnp
	 *
	 * @param \DateTime $date Set date for which the report is generated.
	 * @param string    $type Report type.
	 * @return Pnp
	 * @throws RepositoryException If error.
	 */
	public function get_pnp( \DateTime $date, string $type ): Pnp {
		$params = new GetPnp( $this->auth );
		$params->setDate( $date );
		$params->setType( $type );

		try {
			$pnp = $this->web_api_client->getPnp( $params );
		} catch ( \Exception $e ) {
			throw new RepositoryException( $e, $e->getMessage() );
		}

		return $pnp;
	}
}
